//Version 1.0

(function() {

	if (typeof techw == "undefined") { // すべてをtechwオブジェクトの下にいれる
		techw = {};
	}
	if (typeof techw.resultCollection == "undefined") { //結果格納用配列を作成
		techw.resultCollection = {};
	}
	if (typeof techw.resultCollectionGraph == "undefined") { //過去データ結果格納用配列を作成
		techw.resultCollectionGraph = {};
	}
	if(typeof techw.addDevice == "undefined"){ //techw.addDevice関数を作成
		techw.addDevice = function(xmlName, deviceId){
			if(typeof techw.deviceList == "undefined"){
				techw.deviceList = [];
			}
			techw.deviceList.push({"id":deviceId, "xml":xmlName});
		}
	}
	if(typeof techw.deleteDevice == "undefined"){ //登録したデバイスを削除する関数
		techw.deleteDevice = function(id){
			for(var i=0; i<techw.deviceList.length; i++){
				if(techw.deviceList[i]["id"] == id){
					techw.deviceList.splice(i, 1);
					return true;
				}
			}
			return false;
		}
	}

	techw.time = 0;
	techw.mode = "read";
	techw.stopFlg = false;

	//エラーコードの定義
	techw.nullDataError = 1000;
	techw.noDeviceError = 1001;
	techw.xmlError = 1002;
	techw.internalServerError = 1003;
	techw.invalidValueError = 1004;
	techw.writingProhibitedError = 1005;
	techw.incompleteXMLError = 1006;
	techw.invalidDataNameError = 1007;
	techw.invalidArgumentError = 1008;


	/*
	 * サーバーからデータを取得し、xmlにしたがって変換し、データを配列として返す関数です
	 *  url: 		データを取得するサーバースクリプトのURL
	 *  callback:	処理が終わった後に呼び出される関数 引数は(data, time)のかたち
	 *  error: 		エラー発生時に呼び出される関数
	 */
	techw.getData = function(url, callback, error) {

		var data = {}, currentDevice = 0, counter = 0, idList = "", done = false;

		if(techw.deviceList == null){
			error("No device has been added", techw.noDeviceError); //deviceListに値がない場合はエラー呼び出し
		}

		looper();

		function looper() {
			if(techw.mode == "read"){ //登録されている全ID分のデータを読み込むまでループ
				if (counter >= techw.deviceList.length) {
					techw.mode = "check"; //終わったらmodeをcheckにしてcallbackを呼び出し
					callback(techw.resultCollection, techw.time);
					done = false;
				}else{
					getJSON("read", techw.deviceList[counter]["id"]);
				}
			}else if(techw.mode = "check"){
				if(done){ //終了したらcallbackを呼び出し
					callback(techw.resultCollection, techw.time);
					done = false;
				}else{
					var idList = ""; //すべてのIDを結合してサーバーに送信
					for(var i=0; i<techw.deviceList.length-1; i++){
						idList += techw.deviceList[i]["id"] + ",";
					}
					idList += techw.deviceList[i]["id"];
					getJSON("check",idList);
				}
			}
		}

		function getJSON(mode, deviceId) { // サーバーからデータを取得
			$.get(url, {mode:mode, deviceId:deviceId, lastTime:techw.time},
			//HTTP GETのハンドラ
			function(received, textStatus, jqXHR){
				if (jqXHR.status == 200) { // 受信完了
					if (!received.flg) { //flgがfalseだった場合読み込み失敗
						error(received.msg, techw.internalServerError);
						return;
					} else if ((received.data == null) || (received.data.length == 0)) {
						error("Received Data is Null", techw.nullDataError);
						return;
					}

					techw.time = received.time;
					data = received.data;
					currentDevice = received.deviceId;

					for(var i=0; i<techw.deviceList.length; i++){ //適合するxml名をxmlNameに代入
						if(techw.deviceList[i]["id"] == currentDevice){
							var xmlName = techw.deviceList[i]["xml"];
							break;
						}
					}
					getXML(xmlName);
					return 0;
				} else if (jqXHR.status == 500) {
					error("Internal Server Error", 500);
				} else if (jqXHR.status == 403) {
					error("Forbidden", 403);
				} else if (jqXHR.status == 404) {
					error("Not Found", 404);
				}
			}, "json");
		}

		function getXML(xmlName) { // XMLの読み込み
			$.ajax({
				url : xmlName,
				type : 'GET',
				dataType : 'xml',
				timeout : 3000,
				error :function( xhr, status, errorString ) {
			        if((xhr.status == "404") && (xmlName.indexOf("default.xml") == -1)){ //xmlNameが404の場合はdefault.xmlを読み込み
			        	xmlName = "./default.xml";
			        	getXML();
			        }else{
			        	error("XML Loading Error: " + errorString, techw.xmlError);
			        }
			    },
				success : parseXML
			// ロードに成功したらパーサー関数を呼び出す
			});
		}

		function parseXML(xml) { // XMLに従いデータを変換する
			var string_array = splitStr(data, 2);
			var name, address, length, type, scale, offset, array, elemnts, unit, bit_adress, low_active, radix_point;
			var results = {}, map = {}; // 結果を格納するオブジェクトを宣言;
			for ( var i = 0; i < $(xml).find("field").length; i++) {
				name = $(xml).find("field").eq(i).attr("name");
				address = parseInt($(xml).find("field").eq(i).find("address").text(), 10);
				length = parseInt($(xml).find("field").eq(i).find("length").text(), 10);
				type = $(xml).find("field").eq(i).find("type").text().replace(/\s+/g, "");

				if (name === null || address === null || type === null) { // name,address,length,typeの内1つでも欠けていたらエラー
					array[i] = undefined;
					type = undefined;
				}

				switch (type) {
				case "signed": // 符号付整数の場合
					if (length === null) {
						result[name] = undefined;
						break;
					}
					if ($(xml).find("field").eq(i).find("type").attr("scale") != undefined) {
						scale = parseFloat($(xml).find("field").eq(i).find("type").attr("scale"), 10);
					} else {
						scale = 1;
					}
					if ($(xml).find("field").eq(i).find("type").attr("offset") != undefined) {
						offset = parseFloat($(xml).find("field").eq(i).find("type").attr("offset"), 10);
					} else {
						offset = 0;
					}
					if ($(xml).find("field").eq(i).find("type").attr("array") === "true") {
						array = true;
					} else {
						array = false;
					}
					if ($(xml).find("field").eq(i).find("type").attr("elements") != undefined) {
						elements = parseInt($(xml).find("field").eq(i).find("type").attr("elements"), 10);
					} else {
						array = false;
					}
					if ($(xml).find("field").eq(i).find("type").attr("unit") != undefined) {
						unit = $(xml).find("field").eq(i).find("type").attr("unit");
					} else {
						unit = "V";
					}
					var tmp, str = 0;
					if (array == true) { // arrayがtrueの場合element個の配列として変換する
						results[name] = [];
						var address_offset = Math.ceil(length / 8);
						for ( var j = 0; j < elements; j++) {
							str = returnHex(address + address_offset * j, length, string_array);
							if (str == undefined || str == null) {
								results[name][j] = undefined;
								continue;
							}
							tmp = parseInt(str, 16) >>> (str.length * 4 - length);
							if (tmp & (1 << (length - 1)) != 0) {
								results[name][j] = (~tmp + 1) * scale + offset;
							} else {
								results[name][j] = tmp * scale + offset;
							}
						}
					} else {
						str = returnHex(address, length, string_array);
						if (str == undefined || str == null) {
							results[name] = undefined;
							break;
						}
						tmp = parseInt(str, 16) >>> (str.length * 4 - length);
						if (tmp & (1 << (length - 1)) != 0) {
							results[name] = (~tmp + 1) * scale + offset;
						} else {
							results[name] = tmp * scale + offset;
						}
					}
					break;

				case "unsigned": // 符号なし整数の場合
					if (length === null) {
						results[name] = undefined;
						break;
					}
					if ($(xml).find("field").eq(i).find("type").attr("scale") != undefined) {
						scale = parseFloat($(xml).find("field").eq(i).find("type").attr("scale"), 10);
					} else {
						scale = 1;
					}
					if ($(xml).find("field").eq(i).find("type").attr("offset") != undefined) {
						offset = parseFloat($(xml).find("field").eq(i).find("type").attr("offset"), 10);
					} else {
						offset = 0;
					}
					if ($(xml).find("field").eq(i).find("type").attr("array") === "true") {
						array = true;
					} else {
						array = false;
					}
					if ($(xml).find("field").eq(i).find("type").attr("elements") != undefined) {
						elements = parseInt($(xml).find("field").eq(i).find("type").attr("elements"), 10);
					} else {
						array = false;
					}
					if ($(xml).find("field").eq(i).find("type").attr("unit") != undefined) {
						unit = $(xml).find("field").eq(i).find("type").attr("unit");
					} else {
						unit = "V";
					}
					var str = 0;
					if (array == true) {
						results[name] = [];
						var address_offset = Math.ceil(length / 8);
						for ( var j = 0; j < elements; j++) {
							str = returnHex(address + address_offset * j, length, string_array);
							if (str == undefined || str == null) {
								results[name][j] = undefined;
								continue;
							}
							results[name][j] = (parseInt(str, 16) >> (str.length * 4 - length)) * scale + offset;
						}
					} else {
						str = returnHex(address, length, string_array);
						if (str == undefined || str == null) {
							results[name] = undefined;
							break;
						}
						results[name] = (parseInt(str, 16) >> (str.length * 4 - length)) * scale + offset;
					}
					break;

				case "float": // 32bit浮動小数点数の場合
					if ($(xml).find("field").eq(i).find("type").attr("scale") != undefined) {
						scale = parseFloat($(xml).find("field").eq(i).find("type").attr("scale"), 10);
					} else {
						scale = 1;
					}
					if ($(xml).find("field").eq(i).find("type").attr("offset") != undefined) {
						offset = parseFloat($(xml).find("field").eq(i).find("type").attr("offset"), 10);
					} else {
						offset = 0;
					}
					if ($(xml).find("field").eq(i).find("type").attr("array") === "true") {
						array = true;
					} else {
						array = false;
					}
					if ($(xml).find("field").eq(i).find("type").attr("elements") != undefined) {
						elements = parseInt($(xml).find("field").eq(i).find("type").attr("elements"), 10);
					} else {
						array = false;
					}
					if ($(xml).find("field").eq(i).find("type").attr("unit") != undefined) {
						unit = $(xml).find("field").eq(i).find("type").attr("unit");
					} else {
						unit = "V";
					}
					var tmp, str, exponent, fraction = 0, sign = 1;
					if (array == true) {
						results[name] = [];
						for ( var j = 0; j < elements; j++) {
							str = returnHex(address + j * 4, 32, string_array);
							if (str == undefined || str == null) {
								results[name][j] = undefined;
								continue;
							}
							tmp = parseInt(str, 16);
							if ((tmp & 0x80000000) != 0)
								sign = -1; // 最上位ビットを見て符号を決定
							exponent = ((tmp & 0x7f800000) >> 23) - 127; // 指数部を取り出し
							fraction = (tmp & 0x7fffff) / (Math.pow(2, 23)) + 1; // 仮数部を取り出し
							results[name][j] = sign * fraction * Math.pow(2, exponent) * scale + offset; // float値への換算
							if (tmp == 0) {
								results[name][j] = 0;
							}
						}
					} else {
						str = returnHex(address, 32, string_array);
						if (str == undefined || str == null) {
							results[name][j] = undefined;
							break;
						}
						tmp = parseInt(str, 16);
						if ((tmp & 0x80000000) != 0){
							sign = -1;
						}
						exponent = ((tmp & 0x7f800000) >> 23) - 127;
						fraction = (tmp & 0x7fffff) / (Math.pow(2, 23)) + 1;
						results[name] = sign * fraction * Math.pow(2, exponent) * scale + offset;
						if (tmp == 0) {
							results[name] = 0;
						}
					}
					break;

				case "boolean": // 真偽値の場合
					if (length === null) {
						results[name] = undefined;
						break;
					}
					if ($(xml).find("field").eq(i).find("type").attr("bit_address") != undefined) {
						bit_address = parseInt($(xml).find("field").eq(i).find("type").attr("bit_address"), 10);
					} else {
						bit_address = 0;
					}
					if ($(xml).find("field").eq(i).find("type").attr("low_active") === "true") {
						low_active = true;
					} else {
						low_active = false;
					}
					if ($(xml).find("field").eq(i).find("type").attr("array") === "true") {
						array = true;
					} else {
						array = false;
					}
					if ($(xml).find("field").eq(i).find("type").attr("elements") != undefined) {
						elements = parseInt($(xml).find("field").eq(i).find("type").attr("elements"), 10);
					} else {
						array = false;
					}

					var tmp = parseInt(returnHex(address, length, string_array), 16);
					if (array === true) {
						results[name] = [];
						for ( var j = 0; j < elements; j++) {
							if ((tmp & (1 << j+bit_address)) != 0) {
								results[name][j] = !low_active;
							} else {
								results[name][j] = low_active;
							}
						}
					} else {
						if ((tmp & (1 << bit_address)) != 0) {
							results[name] = !low_active;
						} else {
							results[name] = low_active;
						}
					}
					break;

				case "string": // 文字列の場合
					if ($(xml).find("field").eq(i).find("type").attr("encoding") != undefined) {
						encoding = "utf-8";
					} else {
						encoding = "utf-8";
					}
					if ($(xml).find("field").eq(i).find("type").attr("char_count") != undefined) {
						char_counts = $(xml).find("field").eq(i).find("type").attr("char_count");
					} else {
						char_counts = 1;
					}

					for ( var j = 0; j < char_counts; j++) {
						result[name] += String.fromCharCode(parseInt(returnHex(address + j, 8, string_array), 16));
					}
					break;

				case "fixed": // 固定小数点の場合
					if ($(xml).find("field").eq(i).find("type").attr("scale") != undefined) {
						scale = parseFloat($(xml).find("field").eq(i).find("type").attr("scale"), 10);
					} else {
						scale = 1;
					}
					if ($(xml).find("field").eq(i).find("type").attr("offset") != undefined) {
						offset = parseFloat($(xml).find("field").eq(i).find("type").attr("offset"), 10);
					} else {
						offset = 0;
					}
					if ($(xml).find("field").eq(i).find("type").attr("array") === "true") {
						array = true;
					} else {
						array = false;
					}
					if ($(xml).find("field").eq(i).find("type").attr("elements") != undefined) {
						elements = parseInt($(xml).find("field").eq(i).find("type").attr("elements"), 10);
					} else {
						array = false;
					}
					if ($(xml).find("field").eq(i).find("type").attr("unit") != undefined) {
						unit = $(xml).find("field").eq(i).find("type").attr("unit");
					} else {
						unit = "V";
					}
					if ($(xml).find("field").eq(i).find("type").attr("radix_point") != undefined) {
						radix_point = parseInt($(xml).find("field").eq(i).find("type").attr("radix_point"), 10);
					} else {
						radix_point = 8;
					}

					var tmp, radix_length, str;
					if (array == true) {
						results[name] = [];
						address_offset = Math.ceil(length / 8);
						for ( var j = 0; j < elements; j++) {
							str = returnHex(address + addres_offset * j, length, string_array);
							if (str == undefined || str == null) {
								results[name][j] = undefined;
								continue;
							}
							tmp = parseInt(str, 16);
							radix_length = length - radix_point;
							results[name][j] = tmp >> radix_point;
							for ( var k = 0; k < radix_length; k++) {
								if (tmp & (1 << k)) {
									results[name][j] += 1 / Math.pow(2, radix_length - k);
								}
							}
							if (tmp & (1 << length)) {
								results[name][j] *= -1;
							}
						}
					} else {
						str = returnHex(address, length, string_array);
						if (str == undefined || str == null) {
							results[name] = undefined;
							break;
						}
						tmp = parseInt(str, 16);
						radix_length = length - radix_point
						if (tmp & (1 << length)) {
							sign = -1;
						}
						results[name] = tmp >> radix_point;
						for ( var j = 0; j < radix_length; j++) {
							if (tmp & (1 << j)) {
								results[name] += 1 / Math.pow(2, radix_length - j);
							}
						}
						if (tmp & (1 << length)) {
							results[name] *= -1;
						}
					}
					break;
				}
			}
			techw.resultCollection[currentDevice] = results; // パースが終わったらコールバックの関数を呼び出し
			done = true;
			counter++
			looper();
		}

		function returnHex(address, length, string_array) {
			var hexstring = string_array[address];
			for ( var i = 1; i < Math.ceil(length / 8); i++) {
				hexstring += string_array[address + i];
			}
			return hexstring;
		}

		function splitStr(dataString, cutOffNumber) {
			var data = [];
			var n = 0;
			while (n < dataString.length) {
				data.push(dataString.substring(n, n + cutOffNumber));
				n += cutOffNumber;
			}
			return data;
		}
	}

	techw.getData.getResult = function(){
		return techw.resultCollection;
	}

	techw.getData.restart = function(){
		techw.getData(techw.getData.url, techw.getData.callback, techw.getData.error);
	}


	/*
	 * xmlにしたがってデータを変換し、サーバーにポストする関数です xmlName: XMLのURL url:
	 * データを取得するサーバースクリプトのURL callback: 処理が終わった後に呼び出される関数 error: エラー発生時に呼び出される関数
	 * data: 送信するデータ dataName: XMLに記されているデータの名前 deviceId: データを取得したい機器の機器番号
	 * index: 配列の場合の位置 mask: booleanでマスクビットを使用する場合true
	 */
	techw.postData = function(url, callback, error, data, dataName, deviceId, index, mask) {

		if (index == null) {
			index = 0;
		}
		if (mask == null) {
			mask = false;
		}
		if(typeof data != "object"){
			error("Data is not array", techw.invalidArgumentError);
		}

		if(techw.deviceList == null){
			error("No device has been added", techw.noDeviceError); //deviceListに値がない場合はエラー呼び出し
		}

		for(var i=0; i<techw.deviceList.length; i++){
			if(techw.deviceList[i]["id"] == deviceId){
				getXML(techw.deviceList[i]["xml"]);
				break;
			}
		}

		function getXML(xmlName) { // XMLの読み込み
			$.ajax({
				url : xmlName,
				type : 'GET',
				dataType : 'xml',
				timeout : 3000,
				error :function( xhr, status, errorString ) {
			        if((status == "404") && (xmlName.indexOf("default.xml") == -1)){
			        	getXML( "./default.xml");
			        }else{
			        	error("XML Loading Error: " + errorString, techw.xmlError);
			        }
			    },
				success : parseXML
			// ロードに成功したらパーサー関数を呼び出す
			});
		}

		function parseXML(xml) { // XMLに従いデータを変換する
			var name, address, length, type, scale, offset, array, elemnts, unit, bit_adress, low_active, foundFlg=false;
			for ( var i = 0; i < $(xml).find("field").length; i++) {
				if ($(xml).find("field").eq(i).attr("name") == dataName) { // 機器番号が一致したらその中にdataNameと同じ名前の項目があるか探索
					foundFlg = true;
					address = parseInt($(xml).find("field").eq(i).find("address").text(), 10);
					length = parseInt($(xml).find("field").eq(i).find("length").text(), 10);
					type = $(xml).find("field").eq(i).find("type").text().replace(/\s+/g, "");

					if (name === null || address === null || type === null) { // name,address,length,typeの内1つでも欠けていたらエラー
						array[i] = undefined;
						type = undefined;
					}

					switch (type) {
					case "signed": // 符号付整数の場合
						if (length === null) {
							error("length is null", techw.incompleteXMLError);
							break;
						}
						if ($(xml).find("field").eq(i).find("type").attr("scale") != undefined) {
							scale = parseFloat($(xml).find("field").eq(i).find("type").attr("scale"), 10);
						} else {
							scale = 1;
						}
						if ($(xml).find("field").eq(i).find("type").attr("offset") != undefined) {
							offset = parseFloat($(xml).find("field").eq(i).find("type").attr("offset"), 10);
						} else {
							offset = 0;
						}
						if ($(xml).find("field").eq(i).find("type").attr("array") === "true") {
							array = true;
						} else {
							array = false;
						}
						if ($(xml).find("field").eq(i).find("type").attr("elements") != undefined) {
							elements = parseInt($(xml).find("field").eq(i).find("type").attr("elements"), 10);
						} else {
							array = false;
						}
						if ($(xml).find("field").eq(i).find("type").attr("elements") != undefined) {
							elements = parseInt($(xml).find("field").eq(i).find("type").attr("elements"), 10);
						} else {
							array = false;
						}
						if ($(xml).find("field").eq(i).find("type").attr("direction") == "r") {
							error("Writing is not Allowed", techw.writingProhibitedError);
							return;
						}

						var tmp, rString = "", bits;
						for ( var j = 0; j < data.length; j++) {
							if(typeof data[j] != "number") error("Invalid Argument", techw.invalidValueError);

							tmp = (data[j] - offset) / scale;
							if (tmp < 0) {
								tmp = (~tmp) + 1;
							}
							if ((tmp > ((1 << (length - 1)) - 1)) || (tmp < (2 << (length - 1)) * -1)) { //
								error("Out of memory", techw.invalidValueError);
								tmp = 0;
							}
							rString += zeroPadding(Math.round(tmp).toString(16), Math.ceil(length / 8) * 2)
						}
						ajaxPost(rString, address + index * Math.ceil(length / 8));
						break;

					case "unsigned": // 符号なし整数の場合
						if (length === null) {
							error("length is null", techw.incompleteXMLError);
							break;
						}
						if ($(xml).find("field").eq(i).find("type").attr("scale") != undefined) {
							scale = parseFloat($(xml).find("field").eq(i).find("type").attr("scale"), 10);
						} else {
							scale = 1;
						}
						if ($(xml).find("field").eq(i).find("type").attr("offset") != undefined) {
							offset = parseFloat($(xml).find("field").eq(i).find("type").attr("offset"), 10);
						} else {
							offset = 0;
						}
						if ($(xml).find("field").eq(i).find("type").attr("array") === "true") {
							array = true;
						} else {
							array = false;
						}
						if ($(xml).find("field").eq(i).find("type").attr("elements") != undefined) {
							elements = parseInt($(xml).find("field").eq(i).find("type").attr("elements"), 10);
						} else {
							array = false;
						}
						if ($(xml).find("field").eq(i).find("type").attr("direction") == "r") {
							error("Writing is not Allowed", techw.writingProhibitedError);
							return;
						}

						var tmp, rString = ""; // 結果をストアする文字列
						for ( var j = 0; j < data.length; j++) {
							if(typeof data[j] != "number") error("Invalid Argument", techw.invalidValueError);

							tmp = (data[j] - offset) / scale;
							if (tmp > ((1 << length) - 1)) { // 引数が2^n-1を超えていた場合エラー関数を呼び出し
								error("Out of memory", techw.invalidValueError);
								return;
							}
							rString += zeroPadding(Math.round(tmp).toString(16), Math.ceil(length / 8) * 2);
						}
						ajaxPost(rString, address + index * Math.ceil(length / 8));
						break;

					case "boolean": // 真偽値の場合
						if (length === null) {
							error("length is null", techw.incompleteXMLError);
							break;
						}
						if ($(xml).find("field").eq(i).find("type").attr("bit_address") != undefined) {
							bit_address = parseInt($(xml).find("field").eq(i).find("type").attr("bit_address"), 10);
						} else {
							bit_address = 0;
						}
						if ($(xml).find("field").eq(i).find("type").attr("low_active") === "true") {
							low_active = true;
						} else {
							low_active = false;
						}
						if ($(xml).find("field").eq(i).find("type").attr("array") === "true") {
							array = true;
						} else {
							array = false;
						}
						if ($(xml).find("field").eq(i).find("type").attr("direction") == "r") {
							error("Writing is not Allowed", techw.writingProhibitedError);
							return;
						}

						var tmp = 0, padding;
						if (mask == true) {
							padding = 4;
							for ( var j = 0; j < data.length; j++) {
								if (data[j]) {
									tmp = tmp | ((1 << (j + bit_address + 8 + index)) + (1 << (j + bit_address + index)));
								} else {
									tmp = tmp | (1 << (j + bit_address + 8 + index));
								}
							}
						} else {
							padding = 2;
							for ( var j = 0; j < data.length; j++) {
								if (data[j]) {
									tmp = tmp | (1 << (j + bit_address + index));
								}
							}
						}

						if (low_active) {
							tmp = ~tmp;
						}
						ajaxPost(zeroPadding(tmp.toString(16), padding), address);
						break;

					case "fixed": // 固定小数点の場合
						if ($(xml).find("field").eq(i).find("type").attr("scale") != undefined) {
							scale = parseFloat($(xml).find("field").eq(i).find("type").attr("scale"), 10);
						} else {
							scale = 1;
						}
						if ($(xml).find("field").eq(i).find("type").attr("offset") != undefined) {
							offset = parseFloat($(xml).find("field").eq(i).find("type").attr("offset"), 10);
						} else {
							offset = 0;
						}
						if ($(xml).find("field").eq(i).find("type").attr("array") === "true") {
							array = true;
						} else {
							array = false;
						}
						if ($(xml).find("field").eq(i).find("type").attr("elements") != undefined) {
							elements = parseInt($(xml).find("field").eq(i).find("type").attr("elements"), 10);
						} else {
							array = false;
						}
						if ($(xml).find("field").eq(i).find("type").attr("unit") != undefined) {
							unit = $(xml).find("field").eq(i).find("type").attr("unit");
						} else {
							unit = "V";
						}
						if ($(xml).find("field").eq(i).find("type").attr("radix_point") != undefined) {
							radix_point = parseInt($(xml).find("field").eq(i).find("type").attr("radix_point"), 10);
						} else {
							radix_point = 8;
						}
						if ($(xml).find("field").eq(i).find("type").attr("direction") == "r") {
							error("Writing is not Allowed", techw.writingProhibitedError);
							return;
						}

						var dec, integer, radix_length, rString = "", decBi = 0;
						for ( var j = 0; j < data.length; j++) {
							tmp = (data[j] - offset) / scale;
							integer = parseInt(String(tmp).split(".")[0]);
							dec = parseFloat("0." + (String(tmp)).split(".")[1]);
							radix_length = length - radix_point;
							if (integer > (2 << radix_length) - 1){
								error("Out of memory", techw.invalidValueError);
								return;
							}
							integer = integer << radix_point;
							decBi = 0;
							for ( var k = 1; k <= radix_point; k++) {
								if ((dec - Math.pow(2, -k)) >= 0) {
									decBi += 1 << (radix_point - k);
									dec -= Math.pow(2, -k);
								}
							}
							rString += zeroPadding(((integer + decBi).toString(16)), Math.ceil(length / 8) * 2);
						}
						ajaxPost(rString, address + index * Math.ceil(length / 8));
						break;

					case "float": // 32bit浮動小数点数の場合
						if ($(xml).find("field").eq(i).find("type").attr("scale") != undefined) {
							scale = parseFloat($(xml).find("field").eq(i).find("type").attr("scale"), 10);
						} else {
							scale = 1;
						}
						if ($(xml).find("field").eq(i).find("type").attr("offset") != undefined) {
							offset = parseFloat($(xml).find("field").eq(i).find("type").attr("offset"), 10);
						} else {
							offset = 0;
						}
						if ($(xml).find("field").eq(i).find("type").attr("array") === "true") {
							array = true;
						} else {
							array = false;
						}
						if ($(xml).find("field").eq(i).find("type").attr("elements") != undefined) {
							elements = parseInt($(xml).find("field").eq(i).find("type").attr("elements"), 10);
						} else {
							array = false;
						}
						if ($(xml).find("field").eq(i).find("type").attr("direction") == "r") {
							error("Writing is not Allowed", techw.writingProhibitedError);
							return;
						}

						var wnum, dec, tmp, exp, sign, rString = "";
						for ( var j = 0; j < data.length; j++) {
							tmp = (data[j] - offset) / scale;
							sign = tmp > 0 ? 0 : 1; // 符号ビットを決定
							wnum = tmp.toString(2).split(".")[0]; // 整数部と小数部に分離
							dec = tmp.toString(2).split(".")[1];
							if (tmp > 1) {
								exp = wnum.length - 1; // 指数部の決定
								tmp = tmp * Math.pow(2, -exp) - 1; // ケチ表現に変換
							} else {
								exp = -(dec.indexOf("1") + 1);
								tmp = tmp * Math.pow(2, -exp) - 1;
							}
							if ((exp > 127) || (exp < -127) || (tmp.toString(2).split(".")[1].length > 23)) {
								error("Out of memory", techw.invalidValueError);
								return;
							}
							rString += sign.toString(2) + ("00000000" + (exp + 127).toString(2)).slice(-8)
									+ (tmp.toString(2).split(".")[1] + "00000000000000000000000").slice(0, 23);
						}
						ajaxPost(rString, address + index * 4);
						break;
					}
				}
			}
			if(!foundFlg){
				error("DataName is not found", techw.invalidDataNameError);
			}
		}

		function ajaxPost(data, address) { // サーバに値をポストする関数
			$.post(url, {mode:"public", address:address, data:data, deviceId:deviceId},
				function(received, textStatus, jqXHR){
					if (jqXHR.status == 200) {
						if (received.flg) {
							callback(received.msg, dataName, index);
						} else {
							error(received.msg, techw.internalServerError);
						}
					} else if (jqXHR.status == 500) {
						error("Internal Server Error", 500);
					} else if (jqXHR.status == 403) {
						error("Forbidden", 403);
					} else if (jqXHR.status == 404) {
						error("Not Found", 404);
					}
				}, "json");
		}

		function zeroPadding(data, digits) { // ゼロ埋めをする関数
			return ("00000000" + data).slice(-digits)
		}
	};

	techw.getRecord = function(url, callback, error, start, end, interval, average, aveTime) {

		var data = "0", counter = 0;
		getJSON("log_data", techw.deviceList[counter]["id"]);

		if(techw.deviceList == null){
			error("No device has been added", techw.noDeviceError); //deviceListに値がない場合はエラー呼び出し
		}
		function looper() {
			if (++counter >= techw.deviceList.length ) {
				callback(techw.resultCollectionGraph);
				return 0;
			}else{
				getJSON("log_data", techw.deviceList[counter]["id"]);
				return 0;
			}
		}

		function getJSON(mode, deviceId) {
			$.get(url, {mode:mode, deviceId:deviceId, start:start, end:end, interval:interval},
			//HTTP GETのハンドラ
			function(received, textStatus, jqXHR){
				data = received;
				if (jqXHR.status == 200) { // 受信完了
					if (data == null || data.length == 0) {
						error("Data is Null", techw.nullDataError);
						return;
					}
					getXML(techw.deviceList[counter]["xml"]);
				} else if (jqXHR.status == 500) {
					error("Internal Server Error", 500);
				} else if (jqXHR.status == 403) {
					error("Forbidden", 403);
				} else if (jqXHR.status == 404) {
					error("Not Found", 404);
				}
			}, "json");
		}

		function getXML(xmlName) {
			$.ajax({
				url : xmlName,
				type : 'GET',
				dataType : 'xml',
				timeout : 3000,
				error : function(xhr, status, errorString) {
					if ((status == "404") && (xmlName.indexOf("default.xml") == -1)) {
						getXML("./default.xml");
					} else {
						error("XML Loading Error: " + errorString, techw.xmlError);
					}
				},
				success : parseXML
			// ロードに成功したらパーサー関数を呼び出す
			});
		}

		function parseXML(xml) {
			var name, address, length, type, scale, offset, unit, bit_adress, low_active, incremental, string_array;
			var results = {};
			for ( var i = 0; i < $(xml).find("field").length; i++) {
				name = $(xml).find("field").eq(i).attr("name");
				address = parseInt($(xml).find("field").eq(i).find("address").text(), 10);
				length = parseInt($(xml).find("field").eq(i).find("length").text(), 10);
				type = $(xml).find("field").eq(i).find("type").text().replace(/\s+/g, "");

				if (name === null || address === null || type === null) { // name,address,length,typeの内1つでも欠けていたらエラー
					results[i] = undefined;
					type = undefined;
				}

				switch (type) {
				case "signed":
					if (length === null) {
						result[name] = undefined;
						break;
					}
					if ($(xml).find("field").eq(i).find("type").attr("scale") != undefined) {
						scale = parseFloat($(xml).find("field").eq(i).find("type").attr("scale"), 10);
					} else {
						scale = 1;
					}
					if ($(xml).find("field").eq(i).find("type").attr("offset") != undefined) {
						offset = parseFloat($(xml).find("field").eq(i).find("type").attr("offset"), 10);
					} else {
						offset = 0;
					}
					if ($(xml).find("field").eq(i).find("type").attr("incremental") === "true") {
						incremental = true;
					} else {
						incremental = false;
					}
					if ($(xml).find("field").eq(i).find("type").attr("unit") != undefined) {
						unit = $(xml).find("field").eq(i).find("type").attr("unit");
					} else {
						unit = "V";
					}
					if ($(xml).find("field").eq(i).find("type").attr("array") === "true") {
						array = true;
					} else {
						array = false;
					}
					if ($(xml).find("field").eq(i).find("type").attr("elements") != undefined) {
						elements = parseInt($(xml).find("field").eq(i).find("type").attr("elements"), 10);
					} else {
						array = false;
					}
					var tmp, str, previous = 0;
					results[name] = [];
					function computeArraySigned(address_offset) {
						var array = [];
						if (incremental == true) {
							string_array = splitStr(data[0]["data"], 2);
							str = returnHex(address + address_offset, length, string_array);
							tmp = parseInt(str, 16) >>> (str.length * 4 - length);
							if (tmp & (1 << (length - 1)) != 0) {
								previous = (~tmp + 1) * scale + offset; // 差分をとるために最初の値を決定
							} else {
								previous = tmp * scale + offset;
							}
							for ( var j = 1; j < data.length; j++) {
								string_array = splitStr(data[j]["data"], 2);
								if (string_array == null) {
									array[j - 1] = [ data[j]["time"], null ];
									continue;
								}
								str = returnHex(address + address_offset, length, string_array);
								tmp = parseInt(str, 16) >>> (str.length * 4 - length);
								if (tmp & (1 << (length - 1)) != 0) {
									array[j - 1] = [ data[j]["time"], (~tmp + 1) * scale + offset - previous ]; // 値の差分をとる
									previous = ~(tmp + 1) * scale + offset;
								} else {
									array[j - 1] = [ data[j]["time"], tmp * scale + offset - previous ];
									previous = tmp * scale + offset;
								}
							}
							return array;
						} else {
							var array = [];
							var count = 0, sum = 0, lastTime = data[0]["time"], index = 0;
							for ( var j = 0; j < data.length; j++) {
								string_array = splitStr(data[j]["data"], 2);
								if (string_array == null) {
									array[j] = [ data[j]["time"], null ];
									continue;
								}
								str = returnHex(address + address_offset, length, string_array);
								tmp = parseInt(str, 16) >>> (str.length * 4 - length);
								if (average == true) {
									if (tmp & (1 << (length - 1)) != 0) {
										sum += (~tmp + 1) * scale + offset;
									} else {
										sum += tmp * scale + offset;
									}
									count++;
									if ((data[j]["time"] - lastTime) >= aveTime * 60000) {
										array[index] = [ data[j]["time"], sum / count ]
										sum = 0;
										count = 0;
										lastTime = data[j]["time"];
										index++;
									}
								} else {
									if (tmp & (1 << (length - 1)) != 0) {
										array[j] = [ data[j]["time"], (~tmp + 1) * scale + offset ];
									} else {
										array[j] = [ data[j]["time"], tmp * scale + offset ];
									}
								}
							}
							return array;
						}
					}
					if (array == true) {
						for ( var k = 0; k < elements; k++) {
							results[name][k] = computeArraySigned(Math.ceil(length / 8) * k);
						}
					} else {
						results[name] = computeArraySigned(0);
					}
					break;

				case "unsigned":
					if (length === null) {
						results[name] = undefined;
						break;
					}
					if ($(xml).find("field").eq(i).find("type").attr("scale") != undefined) {
						scale = parseFloat($(xml).find("field").eq(i).find("type").attr("scale"), 10);
					} else {
						scale = 1;
					}
					if ($(xml).find("field").eq(i).find("type").attr("offset") != undefined) {
						offset = parseFloat($(xml).find("field").eq(i).find("type").attr("offset"), 10);
					} else {
						offset = 0;
					}
					if ($(xml).find("field").eq(i).find("type").attr("incremental") === "true") {
						incremental = true;
					} else {
						incremental = false;
					}
					if ($(xml).find("field").eq(i).find("type").attr("elements") != undefined) {
						elements = parseInt($(xml).find("field").eq(i).find("type").attr("elements"), 10);
					} else {
						array = false;
					}
					if ($(xml).find("field").eq(i).find("type").attr("unit") != undefined) {
						unit = $(xml).find("field").eq(i).find("type").attr("unit");
					} else {
						unit = "V";
					}
					if ($(xml).find("field").eq(i).find("type").attr("array") === "true") {
						array = true;
					} else {
						array = false;
					}
					if ($(xml).find("field").eq(i).find("type").attr("elements") != undefined) {
						elements = parseInt($(xml).find("field").eq(i).find("type").attr("elements"), 10);
					} else {
						array = false;
					}
					var str, previous = 0;
					results[name] = [];
					function computeArrayUnsigned(address_offset) {
						var array = []
						if (incremental == true) {
							string_array = splitStr(data[0]["data"], 2);
							str = returnHex(address + address_offset, length, string_array);
							previous = (parseInt(str, 16) >> (str.length * 4 - length)) * scale + offset;
							for ( var j = 1; j < data.length; j++) {
								string_array = splitStr(data[j]["data"], 2);
								if (string_array == null) {
									array[j - 1] = [ data[j]["time"], null ];
									continue;
								}
								str = returnHex(address + address_offset, length, string_array);
								array[j - 1] = [ data[j]["time"], ((parseInt(str, 16) >> (str.length * 4 - length)) * scale + offset) - previous ];
								previous = (parseInt(str, 16) >> (str.length * 4 - length)) * scale + offset;
							}
							return array;
						} else {
							var count = 0, sum = 0, lastTime = data[0]["time"], index = 0;
							for ( var j = 0; j < data.length; j++) {
								if (average == true) { // 平均値を求める場合
									string_array = splitStr(data[j]["data"], 2);
									if (string_array == null) {
										array[j] = [ data[j]["time"], null ];
										continue;
									}
									str = returnHex(address + address_offset, length, string_array);
									sum += (parseInt(str, 16) >> (str.length * 4 - length)) * scale + offset;
									count++;
									if ((data[j]["time"] - lastTime) >= aveTime * 60000) {
										array[index] = [ data[j]["time"], sum / count ]
										sum = 0;
										count = 0;
										lastTime = data[j]["time"];
										index++;
									}
								} else {
									string_array = splitStr(data[j]["data"], 2);
									if (string_array == null) {
										array[j] = [ data[j]["time"], null ];
										continue;
									}
									str = returnHex(address + address_offset, length, string_array);
									if(typeof str == "undefined"){
										array[j] = [ data[j]["time"], null ];
									}else{
										array[j] = [ data[j]["time"], (parseInt(str, 16) >> (str.length * 4 - length)) * scale + offset ];
									}

								}
							}
							return array;
						}
					}
					if (array) {
						for ( var k = 0; k < elements; k++) {
							results[name][k] = computeArrayUnsigned(Math.ceil(length / 8) * k);
						}
					} else {
						results[name] = computeArrayUnsigned(0);
					}
					break;

				case "float":
					if ($(xml).find("field").eq(i).find("type").attr("scale") != undefined) {
						scale = parseFloat($(xml).find("field").eq(i).find("type").attr("scale"), 10);
					} else {
						scale = 1;
					}
					if ($(xml).find("field").eq(i).find("type").attr("offset") != undefined) {
						offset = parseFloat($(xml).find("field").eq(i).find("type").attr("offset"), 10);
					} else {
						offset = 0;
					}
					if ($(xml).find("field").eq(i).find("type").attr("array") === "true") {
						array = true;
					} else {
						array = false;
					}
					if ($(xml).find("field").eq(i).find("type").attr("elements") != undefined) {
						elements = parseInt($(xml).find("field").eq(i).find("type").attr("elements"), 10);
					} else {
						array = false;
					}
					if ($(xml).find("field").eq(i).find("type").attr("unit") != undefined) {
						unit = $(xml).find("field").eq(i).find("type").attr("unit");
					} else {
						unit = "V";
					}
					var tmp, str, exponent, fraction, previous = 0, sign = 1;
					results[name] = [];
					function computeArrayFloat(address_offset) {
						var array = []
						if (incremental == true) {
							string_array = splitStr(data[0]["data"], 2);
							str = returnHex(address + address_offset, 32, string_array);
							tmp = parseInt(str, 16);
							if ((tmp & 0x80000000) != 0){
								sign = -1; // 最上位ビットを見て符号を決定
							}
							exponent = ((tmp & 0x7f800000) >> 23) - 127; // 指数部を取り出し
							fraction = (tmp & 0x7fffff) / (Math.pow(2, 23)) + 1; // 仮数部を取り出し
							previous = sign * fraction * Math.pow(2, exponent) * scale + offset; // float値への換算
							if (tmp == 0) {
								previous = 0;
							}
							for ( var j = 1; j < data.length; j++) {
								string_array = splitStr(data[j]["data"], 2);
								if (string_array == null) {
									array[j - 1] = [ data[j]["time"], null ];
									continue;
								}
								str = returnHex(address + address_offset, 32, string_array);
								tmp = parseInt(str, 16);
								if ((tmp & 0x80000000) != 0)
									sign = -1; // 最上位ビットを見て符号を決定
								exponent = ((tmp & 0x7f800000) >> 23) - 127; // 指数部を取り出し
								fraction = (tmp & 0x7fffff) / (Math.pow(2, 23)) + 1; // 仮数部を取り出し
								if (tmp == 0) {
									array[j - 1] = [ data[i]["time"], 0 - previous ];
									previous = 0;
								} else {
									array[j - 1] = [ data[j]["time"], sign * fraction * Math.pow(2, exponent) * scale + offset - previous ]; // float値への換算
									previous = sign * fraction * Math.pow(2, exponent) * scale + offset
								}
							}
							return array;
						} else {
							var count = 0, sum = 0, lastTime = data[0]["time"], index = 0;
							for ( var j = 0; j < data.length; j++) {
								string_array = splitStr(data[0]["data"], 2);
								if (string_array == null) {
									array[j] = [ data[j]["time"], null ];
									continue;
								}
								str = returnHex(address + address_offset, 32, string_array);
								tmp = parseInt(str, 16);
								if ((tmp & 0x80000000) != 0){
									sign = -1; // 最上位ビットを見て符号を決定
								}
								exponent = ((tmp & 0x7f800000) >> 23) - 127; // 指数部を取り出し
								fraction = (tmp & 0x7fffff) / (Math.pow(2, 23)) + 1; // 仮数部を取り出し
								if (average == true) {
									if (tmp == 0) {
										sum += 0;
									} else {
										sum += sign * fraction * Math.pow(2, exponent) * scale + offset;
									}
									count++;
									if ((data[j]["time"] - lastTime) >= aveTime * 60000) {
										array[index] = [ data[j]["time"], sum / count ]
										sum = 0;
										count = 0;
										lastTime = data[j]["time"];
										index++;
									}
								} else {
									if (tmp == 0) {
										array[j] = [ data[j]["time"], 0 ];
									} else {
										array[j] = [ data[j]["time"], sign * fraction * Math.pow(2, exponent) * scale + offset ];
									}
								}
							}
							return array;
						}
					}
					if (array) {
						for ( var k = 0; k < elements; k++) {
							results[name][k] = computeArrayFloat(4 * k);
						}
					} else {
						results[name] = computeArrayFloat(0);
					}
					break;
				}
			}
			techw.resultCollectionGraph[techw.deviceList[counter]["id"]] = results;
			looper();
		}

		function returnHex(address, length, string_array) {
			var hexstring = string_array[address];
			for ( var i = 1; i < Math.ceil(length / 8); i++) {
				hexstring += string_array[address + i];
			}
			return hexstring;
		}

		function splitStr(dataString, cutOffNumber) {
			var data = [];
			var n = 0;
			if (dataString == null) {
				return null;
			}
			while (n < dataString.length) {
				data.push(dataString.substring(n, n + cutOffNumber));
				n += cutOffNumber;
			}
			return data;
		}

	};

	techw.getRecord.getResult = function(){
		return techw.resultCollectionForGraph;
	};

})();