<?php
include './constants.php';

ini_set("ignore_repeated_errors", 1);
ini_set("ignore_repeated_source", 1);
ini_set("log_erorrs", 0);
session_start();

switch($_GET["mode"]){ //get変数の値によって呼び出す変数を決める
	case "check":
		pending_response(explode(",", $_GET["deviceId"]), strtotime($_GET["lastTime"])); //ロングポーリングを行う関数
		break;

	case "read":
		get_data($_GET["deviceId"]); //データベースの値をそのまま読み出す
		break;

	case "log_list":
		get_list($_GET["deviceId"]); //データベースのログの開始日時と最新日時を読み出す
		break;

	case "log_data":
		get_log($_GET["deviceId"], $_GET["start"], $_GET["end"], $_GET["interval"]); //開始日時、終了日時、間隔を決めてログを読み出す
		break;

	default:
		echo "error";
		break;
}

/*
 * データーベースからデータを取得し、クライアントに送信する関数
 */
function get_data($deviceId){
	global $dsn, $user, $pass;
	$post = Array();

	try{
		$dbh = new PDO($dsn, $user, $pass); //データベースから最新のデータ一件を読み出し
		$dbh->setAttribute(PDO::ATTR_ERRMODE, PDO::ERRMODE_EXCEPTION);

		if($deviceId == 0){
			$sql = "SELECT * FROM `data` ORDER BY `time` DESC LIMIT 1";
			$stmt = $dbh->prepare($sql);
			$stmt->execute();
		}else{
			$sql = "SELECT * FROM `data` WHERE id = ? ORDER BY `time` DESC LIMIT 1";
			$stmt = $dbh->prepare($sql);
			$stmt->execute(array($deviceId));
		}

		$post = $stmt->fetch(PDO::FETCH_ASSOC);

		if($post === null){ //データを読み出せなかった場合
			$data["msg"] = "error occured (database access failed)";
			header( 'Content-Type: text/javascript; charset=utf-8' );
			echo json_encode($data);
			exit();
		}

		$ajaxDataArray["data"] = $post["data"]; //データと投稿日時をJSON形式でクライアントに送信
		$ajaxDataArray["time"] = date("Y/m/d H:i:s", $post["time"]);
		$ajaxDataArray["deviceId"] = $deviceId;
		$ajaxDataArray["mode"] = "current-value";
		$ajaxDataArray["flg"] = true;
		header( 'Content-Type: text/javascript; charset=utf-8' );
		echo json_encode($ajaxDataArray);

	}catch(PDOException $e){ //例外処理
		$data["msg"] = "error occured (" . $e->getMessage() . ")";
		$data["flg"] = false;
		header( 'Content-Type: text/javascript; charset=utf-8' );
		echo json_encode($data);
		exit();
	}
}

/*
 * 最新のデータがデータベースになかった場合、機器がデータを送信してくるまで待機
 *
 * $deviceId		データを取得したい機器の番号
 * $lastTime			クライアントが最後に受け取ったデータの日時
 */
function pending_response($deviceList, $lastTime){
	global $dsn, $user, $pass;

	checkDB($deviceList, $lastTime); //クライアントのデータが最新かどうかのチェック

	set_time_limit(TW_CLIENT_TIMEOUT);
	session_write_close(); //ほかのコネクションに影響が及ぶため一度セッションを閉じる

	$socket = socket_create(AF_INET, SOCK_DGRAM, SOL_UDP);
	$port = find_port($socket, $deviceList); //ソケットを作成し、バインドできたポートをDBに登録

	socket_set_option($socket,SOL_SOCKET,SO_RCVTIMEO,array("sec"=>TW_CLIENT_TIMEOUT,"usec"=>0)); //ソケットのタイムアウトを設定

	if(!@socket_recvfrom($socket, $msg, 1024, 0, $from, $comingPort)){ //タイムアウトした場合
		$dbh = new PDO($dsn, $user, $pass);
		$sql = "DELETE FROM `port_client` WHERE port = ?";
		$stmt = $dbh->prepare($sql);
		$stmt->execute(array($port)); //DBからポート番号を削除して終了
		$data["msg"] = "error occured (UDP socket timeout)";
		$data["flg"] = false;
		header( 'Content-Type: text/javascript; charset=utf-8' );
		echo json_encode($data);
		exit();
	}

	socket_close($socket); //接続があったらソケットを閉じ、ポート番号をDBから削除し、get_data関数を呼び出す
	get_data($msg);
	exit();
}

function checkDB($deviceList, $lastTime){
	global $dsn, $user, $pass;
	$post = Array();

	try{
		$dbh = new PDO($dsn, $user, $pass); //データベースから最新のデータを読み出し
		$dbh->setAttribute(PDO::ATTR_ERRMODE, PDO::ERRMODE_EXCEPTION);

		//$deviseListに0が入っていたら単に最新データの日時と比較
		if((is_array($deviceList) && array_search( '0', $deviceList) !== false)/* || $deviceList === '0'*/){
			$sql = "SELECT * FROM `data` ORDER BY `time` DESC LIMIT 1";
			$stmt = $dbh->prepare($sql);
			$stmt->execute();
			$post = $stmt->fetch(PDO::FETCH_ASSOC);
			if($post["time"] > $lastTime){
				$deviceId = "0";
			}
		//$deviceListの中から最新のデータがあるdeviceIdを見つけ、クライアントが送信した日時と比較
		}else{
			$sql = "SELECT * FROM `data` WHERE id = ? ORDER BY `time` DESC LIMIT 1";
			$stmt = $dbh->prepare($sql);
			for($i=0; $i<count($deviceList); $i++){
				$stmt->execute(array($deviceList[$i]));
				$post = $stmt->fetch(PDO::FETCH_ASSOC);
				if($post["time"] > $lastTime){
					$deviceId = $deviceList[$i];
					$lastTime = $post["time"];
				}
			}
		}
	}catch(PDOException $e){ //例外処理
		$data["msg"] = "error occured (" . $e->getMessage() . ")";
		$data["flg"] = false;
		header( 'Content-Type: text/javascript; charset=utf-8' );
		echo json_encode($data);
		exit();
	}

	if(isset($deviceId)){ //クライアントが持っている最新データよりデータベースのデータのほうが新しかったら、そちらをクライアントに送信
		get_data($deviceId);
		exit();
	}else{
		return;
	}
}

/*
 * ログの開始日時と最新日時を返す関数
 */
function get_list($deviceId){
	global $dsn, $user, $pass;
	$post = array();

	try{
		$dbh = new PDO($dsn, $user, $pass);
		$dbh->setAttribute(PDO::ATTR_ERRMODE, PDO::ERRMODE_EXCEPTION);
		$sql = "SELECT * FROM `data_log` WHERE id = ? ORDER BY `time` DESC LIMIT 1"; //ログのテーブルから最新のデータを読み出し
		$stmt = $dbh->prepare($sql);
		$stmt->execute(array($deviceId));
		$post = $stmt->fetch(PDO::FETCH_ASSOC);
		$end = date("Y/m/d H:i:s", $post["time"]); //その日時を送信

		$sql = "SELECT * FROM `data_log` WHERE id = ? ORDER BY `time` ASC LIMIT 1"; //ログのテーブルから最古のデータを読み出し
		$stmt = $dbh->prepare($sql);
		$stmt->execute(array($deviceId));
		$post = $stmt->fetch(PDO::FETCH_ASSOC);
		$start = date("Y/m/d H:i:s", $post["time"]); //その日時を送信

		header( 'Content-Type: text/javascript; charset=utf-8' );
		echo json_encode(array("start" => $start, "end" => $end, "interval" => TW_LOGGING_INTERVAL)); //

	}catch(PDOException $e){
		$data["msg"] = "error occured (" . $e->getMessage() . ")";
		$data["flg"] = false;
		header( 'Content-Type: text/javascript; charset=utf-8' );
		echo json_encode($data);
		exit();
	}
}

/*
 * ログを返す関数
 *
 * $deviceId		機器番号
 * $start			取得開始日時
 * $end				取得終了日時
 * $interval		データを取得する間隔(分単位)
 */
function get_log($deviceId, $start, $end, $interval){
	global $dsn, $user, $pass;
	$post = array();
	$data = array();

	try{
		$dbh = new PDO($dsn, $user, $pass);
		$dbh->setAttribute(PDO::ATTR_ERRMODE, PDO::ERRMODE_EXCEPTION);

		if($deviceId == 0){
			$sql = "SELECT * FROM `data_log` WHERE time BETWEEN ? AND ? ORDER BY `time` ASC";
			$stmt = $dbh->prepare($sql);
			$stmt->execute(array((strtotime($start)-$interval*60), strtotime($end)));
		}else{
			$sql = "SELECT * FROM `data_log` WHERE id = ? AND time BETWEEN ? AND ? ORDER BY `time` ASC";
			$stmt = $dbh->prepare($sql);
			$stmt->execute(array($deviceId, (strtotime($start)-$interval*60), strtotime($end)));
		}

		$post = $stmt->fetch(PDO::FETCH_ASSOC);
		$last = 0;

		while($post = $stmt->fetch(PDO::FETCH_ASSOC)){
			if(abs($post["time"] - $last) >= $interval*60){
				/*if((abs($post["time"] - $last) >= $interval*60*2)&&($last != 0)){
					$data[] = array("time" => ($last+$interval*60)*1000, "data" => null);
				}*/
				$data[] = array("time" => $post["time"]*1000, "data" => $post["data"]);
				$last = $post["time"];
			}
		}

		header( 'Content-Type: text/javascript; charset=utf-8' );
		$post["mode"] = "log";
		echo json_encode($data);

	}catch(PDOException $e){
		$data["msg"] = "error occured (" . $e->getMessage() . ")";
		$data["flg"] = false;
		header( 'Content-Type: text/javascript; charset=utf-8' );
		echo json_encode($data);
		exit();
	}
}

function get_file_data($deviceId, $startStr, $endStr, $interval){
	$temp = array();
	$data = array();
	$start = strtotime($startStr);
	$end = strtotime($endStr);
	$last = 0;
	if($handle = fopen("hoge.csv", "r") === false){
		$data["msg"] = "Cannot open file";
		$data["flg"] = false;
		header( 'Content-Type: text/javascript; charset=utf-8' );
		echo json_encode($data);
		exit();
	}

	while($temp = fgetcsv($handle, ",")){
		if(($temp[0]<start)&&(abs($temp[0] - $last) >= $interval*60)){
			$data[] = array("time" => $temp[0]*1000, "data" => $tempt[1]);
			$last = $post["time"];
		}
	}

	header( 'Content-Type: text/javascript; charset=utf-8' );
	echo json_encode($data);
}

/*
 * $socket			ポートにバインドしたいソケットオブジェクト
 * $deviceList		DBに登録する待ち受け中の機器番号
 * @return			バインドできたポート番号
 */
function find_port(&$socket, $deviceList){ //バインドできるポートを探す関数
	global $dsn, $user, $pass;
	socket_bind($socket, 'localhost');
	if(socket_getsockname($socket, $addr, $port)){
		try{
			$dbh = new PDO($dsn, $user, $pass);
			$sql = "INSERT INTO `port_client` (`id`, `port`) VALUES ( ?, ?)";
			$stmt = $dbh->prepare($sql);
			for($i=0; $i<count($deviceList); $i++){
				$stmt->execute(array($deviceList[$i], $port));
			}
			unset($dbh);
			return $port;
		}catch(PDOException $e){
			$data["msg"] = "error occured (" . $e->getMessage() . ")";
			$data["flg"] = false;
			header( 'Content-Type: text/javascript; charset=utf-8' );
			echo json_encode($data);
			exit();
		}
	}else{
		$data["msg"] = "error occured (socket bind failed)";
		$data["flg"] = false;
		header( 'Content-Type: text/javascript; charset=utf-8' );
		echo json_encode($data);
		exit();
	}

	return -1;
}