<?php
include './constants.php';

ini_set( "display_errors", "0");
session_start();

if($_POST["mode"] == "public"){
	public_command_write($_POST["address"], $_POST["data"], $_POST["deviceId"]);
}else if($_POST["mode"] == "private"){
	private_command_write($_POST["address"], $_POST["data"], $_POST["deviceId"]);
}

/*
 * クライアントから送られてきたコマンドをDBに保存する関数
 * $address			書き込む機器の内部アドレス
 * $data			機器に送信するデータ
 * $deviceId		送信先機器の番号
 */
function public_command_write($address, $data, $deviceId){
	global $dsn, $user, $pass;
	$notify = array();
	$timeLeft = user_management($deviceId);
	if($timeLeft === true){
		try{
			$dbh = new PDO($dsn, $user, $pass);
			$dbh->setAttribute(PDO::ATTR_ERRMODE, PDO::ERRMODE_EXCEPTION);
			$sql = "INSERT INTO `order` (`id`, `address`, `time`, `data`) VALUES ( ?, ?, ?, ?)";
			$stmt = $dbh->prepare($sql);
			$stmt->execute(array($deviceId, $address, time(), $data)); //送信先機器の機器番号、コマンド、データを保存

			$notify["mode"] = "public";
			send_message_device(json_encode($notify), $deviceId); //コマンドを待っているスクリプトにメッセージを送信

			if(command_acknowledgement($dbh->lastInsertId())){ //コマンドが正常に機器に送信されたか確認
				$msg["flg"] = true;
				$msg["msg"] = "OK";
			}else{
				$msg["flg"] = false;
				$msg["msg"] = "Request from device is not found";
			}
			header( 'Content-Type: text/javascript; charset=utf-8' );
			echo json_encode($msg);
			unset($dbh);

		}catch(PDOException $e){
			$data["flg"] = false;
			$data["msg"] = "error occured (" . $e->getMessage() . ")";
			header( 'Content-Type: text/javascript; charset=utf-8' );
			echo json_encode($data);
			exit();
		}
	}else{
		$msg["time"] = date('Y-m-d H:i:s');
		$msg["flg"] = false;
		$msg["msg"] = "Another user have control of this device. Wait for a while.";
		$msg["left"] = $timeLeft;
		header( 'Content-Type: text/javascript; charset=utf-8' );
		echo json_encode($msg);
	}
}

/*
 * コマンドを待っている機器側のスクリプトにメッセージを送信する関数
 * $msg				送信するメッセージ
 * $deviceId		送信先機器の番号
 */
function send_message_device($msg, $deviceId){
	global $dsn, $user, $pass;
	$len = strlen($msg);
	try{
		$dbh = new PDO($dsn, $user, $pass);
		$dbh->setAttribute(PDO::ATTR_ERRMODE, PDO::ERRMODE_EXCEPTION);

		$sql = "SELECT * FROM `port_lanx` WHERE id = ? ORDER BY `no` DESC"; //機器番号が一致するレコードをすべて取り出す
		$stmt = $dbh->prepare($sql);
		$stmt->execute(array($deviceId));

		$post = $stmt->fetch(PDO::FETCH_ASSOC);
		$socket = socket_create(AF_INET, SOCK_DGRAM, SOL_UDP);
		socket_sendto($socket, $msg, $len, 0, 'localhost', $post["port"]); //最新のリクエストには正常なメッセージを送信
		socket_close($socket);

		while($post = $stmt->fetch(PDO::FETCH_ASSOC)){
			$altmsg["mode"] = "exit";
			$len = strlen(json_encode($altmsg));
			$socket = socket_create(AF_INET, SOCK_DGRAM, SOL_UDP);
			socket_sendto($socket, json_encode($altmsg), $len, 0, '127.0.0.1', $post["port"]); //残りのリクエストにはプロセスを終了させるメッセージを送信
			socket_close($socket);
		}

		$sql = "DELETE FROM `port_lanx` WHERE id = ?";
		$stmt = $dbh->prepare($sql);
		$stmt->execute(array($deviceId));

		unset($dbh);
		return 0;
	}catch(PDOException $e){
		$data["flg"] = false;
		$data["msg"] = "error occured (" . $e->getMessage() . ")";
		header( 'Content-Type: text/javascript; charset=utf-8' );
		echo json_encode($data);
		exit();
	}
}

/*
 * 複数のクライアントが同時にコマンドを送信してきた時の処理
 * $deviceId			対象の機器の番号
 */
function user_management($deviceId){
	global $dsn, $user, $pass;
	try{
		$dbh = new PDO($dsn, $user, $pass);
		$dbh->setAttribute(PDO::ATTR_ERRMODE, PDO::ERRMODE_EXCEPTION);
		$sql = "SELECT * FROM `user` WHERE device = ? LIMIT 1";
		$stmt = $dbh->prepare($sql);
		$stmt->execute(array($deviceId));
		$post = $stmt->fetch(PDO::FETCH_ASSOC);

		if($post == null){ //ユーザー管理テーブルにコマンド送信先機器の機器番号が登録されていない場合
			$sql = "INSERT INTO `user` (`user`, `device`, `time`) VALUES ( ?, ?, ?)";
			$stmt = $dbh->prepare($sql);
			$stmt->execute(array(session_id(), $deviceId, time())); //機器番号とクライアントのセッションIDをDBに登録
			unset($dbh);
			return true;

		}elseif($post["user"] == session_id()){ //機器番号が登録されているが、クライアントが同じであった場合
			$sql = "UPDATE `user` SET time = ? WHERE device = ?";
			$stmt = $dbh->prepare($sql);
			$stmt->execute(array(time(), $deviceId)); //DBのレコードの登録日時部分を更新
			unset($dbh);
			return true;

		}elseif((time() - $post["time"]) > TW_DOMINANT_TIME){ //機器番号が登録されていて、クライアントも別だが、最後に登録されてから2分以上前の場合
			$sql = "UPDATE `user` SET user = ? ,time = ? WHERE device = ?";
			$stmt = $dbh->prepare($sql);
			$stmt->execute(array(session_id(), time(), $deviceId)); //クライアントのセッションIDと登録日時を更新
			unset($dbh);
			return true;

		}else{
			return TW_DOMINANT_TIME - (time() - $post["time"]);
		}
	}catch(PDOException $e){
		$data["flg"] = false;
		$data["msg"] = "error occured (" . $e->getMessage() . ")";
		header( 'Content-Type: text/javascript; charset=utf-8' );
		echo json_encode($data);
		exit();
	}
}

function command_acknowledgement($num){
	global $dsn, $user, $pass;
	for($i=0; $i < TW_MAX_COMMAND_WAIT_TIME * 20; $i++){
		try{
			$dbh = new PDO($dsn, $user, $pass);
			$dbh->setAttribute(PDO::ATTR_ERRMODE, PDO::ERRMODE_EXCEPTION);
			$sql = "SELECT * FROM `order` WHERE no = ? LIMIT 1";
			$stmt = $dbh->prepare($sql);
			$stmt->execute(array($num));
			$post = $stmt->fetch(PDO::FETCH_ASSOC);
			if($post["checked"] == true){
				$sql = "DELETE FROM `order` WHERE no = ?";
				$stmt = $dbh->prepare($sql);
				$stmt->execute(array($num)); //命令を削除して終了
				return true;
			}
		}catch(PDOException $e){
			$data["msg"] = "error occured (" . $e->getMessage() . ")";
			$data["flg"] = false;
			header( 'Content-Type: text/javascript; charset=utf-8' );
			echo json_encode($data);
			exit();
		}
		usleep(50000);
	}
	$dbh = new PDO($dsn, $user, $pass);
	$dbh->setAttribute(PDO::ATTR_ERRMODE, PDO::ERRMODE_EXCEPTION);
	$sql = "DELETE FROM `order` WHERE no = ?";
	$stmt = $dbh->prepare($sql);
	$stmt->execute(array($num)); //命令を削除して終了
	return false;
}
?>