<?php
include_once './constants.php';

date_default_timezone_set("Asia/Tokyo");
session_start();

//error_log(print_r($_POST, TRUE));
//error_log(print_r(getallheaders(), TRUE));

if(!isset($_POST["time"])){
	$time = time(); //post変数timeが存在しなかった場合サーバーの時刻を用いる
}else{
	if(strtotime($_POST["time"]) <= 0){
		$time = time();
	}else{
		$time = strtotime($_POST["time"]);
	}
}


write_data($_POST["data"], $_POST["from"], $time);

/*
 *  データベースにデータを格納する関数
 * data:		機器から送られてきた文字列データ
 * deviceId:	機器の機器番号
*/
function write_data($data, $deviceId, $time){
	global $dsn, $user, $pass;

	try{
		$dbh = new PDO($dsn, $user, $pass);

		$sql = "INSERT INTO `data` (`id`, `time`, `data`) VALUES ( ?, ?, ?)"; //送られてきたデータベースに保存
		$stmt = $dbh->prepare($sql);
		$stmt->execute(array($deviceId, $time, $data));

		$sql = "SELECT COUNT(*) FROM `data` WHERE id = ?"; //データべースに保存されている件数を取得
		$stmt = $dbh->prepare($sql);
		$stmt->execute(array($deviceId));
		$count = $stmt->fetchColumn();

		if($count > TW_MAX_TEMPORARY_DATA){	//TW_MAX_TEMPORARY_DATAの数を超えていた場合
			$sql = "SELECT * FROM `data` WHERE id = ? ORDER BY `time` ASC LIMIT 1"; //データベースから一番古いデータを取り出す
			$stmt = $dbh->prepare($sql);
			$stmt->execute(array($deviceId));
			$post = $stmt->fetch(PDO::FETCH_ASSOC);

			$sql = "DELETE FROM `data` WHERE no = ?"; //そのデータを削除
			$stmt = $dbh->prepare($sql);
			$stmt->execute(array($post["no"]));
		}

		store_data($data, $deviceId, $time); //ログを保存する関数を呼び出し
		send_message_client("received", $deviceId); //クライアント側のスクリプトにメッセージを送る関数を呼び出し
	}catch(PDOException $e){
		echo $e->getMessage();
	}

	header( 'Content-Type: text/javascript; charset=utf-8' );
	echo '<!DOCTYPE html\n   PUBLIC "-//W3C//DTD XHTML 1.0 Strict//EN"\n   "http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd">';
	echo "<!-- WebDAC\r\ncmd=10+publish&rslt=10+OK&id=0x00000000&sid=" . urlencode("id=" . session_id()) . "\r\n-->";
}

function store_data($data, $deviceId, $time){ //ログを保存する関数
	global $dsn, $user, $pass;
	$dbh = new PDO($dsn, $user, $pass);
	$sql = "SELECT * FROM `data_log` WHERE id = ? ORDER BY `time` DESC LIMIT 1"; //データベースから最新のデータを読み出し
	$stmt = $dbh->prepare($sql);
	$stmt->execute(array($deviceId));
	$post = $stmt->fetch(PDO::FETCH_ASSOC);

	if(!$post){ //データが存在しない場合はログのテーブルにデータを保存
		$sql = "INSERT INTO `data_log` (`id`, `time`, `data`) VALUES ( ?, ?, ?)";
		$stmt = $dbh->prepare($sql);
		$stmt->execute(array($deviceId, $time, $data));
	}elseif(abs($time - $post["time"]) >= TW_LOGGING_INTERVAL){//前のデータから一定時間以上経過していたらデータベースにデータを保存
		$sql = "INSERT INTO `data_log` (`id`, `time`, `data`) VALUES ( ?, ?, ?)";
		$stmt = $dbh->prepare($sql);
		$stmt->execute(array($deviceId, $time, $data));
	}

	unset($dbh);
	return;
}

//メッセージを送信した後、DBからポート番号を削除する関数
function delete_port($no){
	global $dsn, $user, $pass;
	try {
		$dbh = new PDO($dsn, $user, $pass);
		$sql = "DELETE FROM `port_client` WHERE no = ?";
		$stmt = $dbh->prepare($sql);
		$stmt->execute(array($no));
	} catch (PDOException $e) {
		echo $e->getMessage();
		return -1;
	}
}

//データを待っているクライアント側スクリプトにメッセージを送信する関数
function send_message_client($msg, $deviceId){
	global $dsn, $user, $pass;
	try {
		$dbh = new PDO($dsn, $user, $pass);
		$sql = "SELECT * FROM `port_client` WHERE id = ?";
		$stmt = $dbh->prepare($sql);
		$stmt->execute(array($deviceId));

		$msg = $deviceId;
		$len = strlen($msg);
		while($post = $stmt->fetch(PDO::FETCH_ASSOC)){ //DBに登録されているポート番号すべてに通知を送信
			$socket = socket_create(AF_INET, SOCK_DGRAM, SOL_UDP);
			socket_sendto($socket, $msg, $len, 0, 'localhost', $post["port"]);
			socket_close($socket);
			delete_port($post["no"]);
		}

		unset($post);

		$sql = "SELECT * FROM `port_client` WHERE id = 0"; //機器番号が0の場合は
		$stmt = $dbh->prepare($sql);
		$stmt->execute();

		$msg = '0';
		$len = strlen($msg);
		while($post = $stmt->fetch(PDO::FETCH_ASSOC)){
			$socket = socket_create(AF_INET, SOCK_DGRAM, SOL_UDP);
			socket_sendto($socket, $msg, $len, 0, 'localhost', $post["port"]);
			socket_close($socket);
			delete_port($post["no"]);
		}

		return 0;
	} catch (PDOException $e) {
		return -1;
	}
}