/*
*
* ========================================================================================
*
*   ProgressBar     プログレスバーを描画するクラスです
*
*   length:		   プログレスバーの横の長さ
*   height:         プログレスバーの縦の長さ
*   minimam:        プログレスバーの最小値
*   maximam:        プログレスバーの最大値
*/
function ProgressBar(length, height, minimam, maximam) { // コンストラクタ
	this.length = length;
	this.height = height;
	this.minimam = minimam;
	this.maximam = maximam;

	// デフォルト値を設定
	this.color = 'RGB(0,255,51)';
	this.value = this.maximam / 2;
	this.scale = false;
}

ProgressBar.prototype.setValue = function(value) { // 値を設定するメソッド
	if(value > this.maximam){
		this.value = this.maximam;
	}else if(value < this.minimam){
		this.value = this.minimam;
	}else{
		this.value = value;
	}
};

/*
 * color: 	色を指定("rgb(0, 0, 0)"のように)
 */
ProgressBar.prototype.setColor = function(color) { // 色を設定するメソッド
	this.color = color;
};

/*
 * color: 	色を指定("rgb(0, 0, 0)"のように)
 */
ProgressBar.prototype.setStrokeColor = function(color) { // ストローク色を設定するメソッド
	this.strokeColor = color;
};

/*
 * font:		目盛の文字のフォント
 * unit:		目盛の単位を指定
 */
ProgressBar.prototype.setScale = function(font, unit) { // 目盛を設定するメソッド
	this.font = font;
	this.unit = unit;
	this.scale = true; // このメソッドが呼ばれた場合のみ目盛を描画
};

/*
 * ctx:		canvasの2Dコンテキスト
 * pX:		オブジェクトの右上のx座標
 * pY:		オブジェクトの右上のy座標
 * color:   背景色
 */
ProgressBar.prototype.drawBGColor = function(ctx, pX, pY, color) { // 背景色を設定するメソッド
	// 内側の矩形を描画
	var innerLength = this.length - 2;
	ctx.save();
	ctx.fillStyle = color;
	ctx.fillRect(pX + 1, pY + 1, innerLength, this.height - 2);
	ctx.restore();
}

/*
 * ctx:		canvasの2Dコンテキスト
 * pX:		オブジェクトの右上のx座標
 * pY:		オブジェクトの右上のy座標
 */
ProgressBar.prototype.stroke = function(ctx, pX, pY) { // 描画するメソッド
	// 外側の矩形を描画
	ctx.save();
	ctx.lineWidth = 2;
    ctx.strokeStyle = this.strokeColor;
	ctx.strokeRect(pX, pY, this.length, this.height);
	ctx.restore();

	// 内側の矩形を描画
	var innerLength = (this.value - this.minimam) / (this.maximam - this.minimam) * (this.length - 2);
	ctx.save();
	ctx.fillStyle = this.color;
	ctx.fillRect(pX + 1, pY + 1, innerLength, this.height - 2);
	ctx.restore();

	// 目盛を描画
	if (this.scale) {
		lineLength = 5;
		offsetX = 1;
		offsetY = 5;

		ctx.save();
		ctx.beginPath();
		ctx.lineWidth = 2;
		ctx.moveTo(pX, pY + this.height);
		ctx.lineTo(pX, pY + this.height + lineLength);
		ctx.moveTo(pX + Math.round(this.length / 2), pY + this.height);
		ctx.lineTo(pX + Math.round(this.length / 2), pY + this.height + lineLength);
		ctx.moveTo(pX + this.length, pY + this.height);
		ctx.lineTo(pX + this.length, pY + this.height + lineLength);
		ctx.stroke();
		ctx.restore();

		ctx.font = this.font;
		ctx.fillText(this.minimam + this.unit, pX + offsetX, pY + this.height + lineLength + offsetY);
		ctx.fillText((this.maximam - this.minimam) / 2 + this.minimam + this.unit, pX + Math.round(this.length / 2) + offsetX, pY + this.height + lineLength + offsetY);
		ctx.fillText(this.maximam + this.unit, pX + this.length + 1, pY + this.height + lineLength + offsetY);
	}
};


/*
*
* ========================================================================================
*
*   Indicator    ランプを描画するクラス
*
*/

/*
 * radius:		半径を指定
 */
function Indicator(radius) {
	this.radius = radius;
	this.onColor = 'RGB(0,0,255)';
	this.offColor = 'rgb(255, 255, 255)';
}
// オン時の色を設定するメソッド
Indicator.prototype.setOnColor = function(color) {
	this.onColor = color;
};

// オフ時の色を設定するメソッド
Indicator.prototype.setOffColor = function(color) {
	this.offColor = color;
};

// ストローク色を設定するメソッド
Indicator.prototype.setStrokeColor = function(color) {
	this.strokeColor = color;
};

/*
 * オン時の描画を行うメソッド
 * ctx:		描画するcanvas要素のコンテクスト
 * pX:		描画するオブジェクトの中心のX座標
 * pY:		描画するオブジェクトの中心のY座標
 */
Indicator.prototype.strokeOn = function(ctx, pX, pY) {
    
    // 外側の円を描画します
	ctx.save();
	ctx.beginPath();
	ctx.lineWidth = 1;
	ctx.arc(pX, pY, this.radius, 0, Math.PI * 2, true);
    ctx.strokeStyle = this.strokeColor;
	ctx.stroke();
	ctx.restore();

    // 点灯部の縁を描画します
	ctx.save();
	ctx.beginPath();
	ctx.lineWidth = 1;
	ctx.arc(pX, pY, this.radius - 3, 0, Math.PI * 2, true);
	ctx.stroke();
	ctx.restore();

    // 点灯部を描画します
	ctx.save();
	ctx.beginPath();
	ctx.fillStyle = this.onColor;
	ctx.arc(pX, pY, this.radius - 4, 0, Math.PI * 2, true);
	ctx.fill();
	ctx.restore();
};

/*
 * オフ時の描画を行うメソッド
 * ctx:		描画するcanvas要素のコンテクスト
 * pX:		描画するオブジェクトの中心のX座標
 * pY:		描画するオブジェクトの中心のY座標
 */
Indicator.prototype.strokeOff = function(ctx, pX, pY) {

    ctx.strokeStyle = this.strokeColor;

/*
    // dummy
    // 外側の円を描画します
    ctx.save();
	ctx.beginPath();
	ctx.lineWidth = 1;
	ctx.arc(pX, pY, this.radius, 0, Math.PI * 2, true);
	ctx.stroke();
	ctx.restore();
*/

    // 点灯部の縁を描画します
	ctx.save();
	ctx.beginPath();
	ctx.lineWidth = 1;
    ctx.fillStyle = "#fff";
	ctx.arc(pX, pY, this.radius - 3, 0, Math.PI * 2, true);
    ctx.fill();
	ctx.stroke();
	ctx.restore();

    // 点灯部を描画します
	ctx.save();
	ctx.beginPath();
	ctx.fillStyle = this.offColor;
	ctx.arc(pX, pY, this.radius - 3, 0, Math.PI * 2, true);
	ctx.fill();
	ctx.restore();
};


/*
*
* ========================================================================================
*
*   Switch    スイッチを描画するクラスのコンストラクタ
*
*   ctx         描画するcanvas要素のコンテクスト
*   pX          描画するオブジェクトの右上のX座標
*   pY          描画するオブジェクトの右上のY座標
*   width		描画するオブジェクトの横幅
*   height      描画するオブジェクトの高さ
*/
function Switch(pX, pY, width, height, ctx) {
	this.pX = pX;
	this.pY = pY;
	this.height = height;
	this.width = width;
	this.ctx = ctx;

	this.checked = false;
	this.onColor = 'rgba(0, 0, 255, 0.5)';
	this.offColor = 'rgba(255, 255, 255, 0.5)';
}

//オン時の色を設定するメソッド
Switch.prototype.setOnColor = function(onColor) {
	this.onColor = onColor;
};

//オフ時の色を設定するメソッド
Switch.prototype.setOffColor = function(offColor) {
	this.offColor = offColor;
};

// クリックイベントリスナー
Switch.prototype.onClick = function(e, item, form) {
	var rect = e.target.getBoundingClientRect();
	mouseX = Math.floor(e.clientX - rect.left);
	mouseY = Math.floor(e.clientY - rect.top);
	if (((mouseX > this.pX) && (mouseX < this.pX + this.width)) && ((mouseY > this.pY) && (mouseY < this.pY + this.height))) {
		if (this.checked) { //このオブジェクトがクリックされた場合、formの値を書き換え送信
			this.draw0();
			item.value = 0;
			form.submit();
		} else {
			this.draw1();
			item.value = 1;
			form.submit();
		}
	}
};

// 状態０のときの描写を行うメソッドです
Switch.prototype.draw0 = function() {
	// 前のスイッチを除去
	this.ctx.clearRect(this.pX, this.pY, this.width, this.height);

	this.ctx.save();
	this.ctx.beginPath();
	this.ctx.lineWidth = 1;
	this.ctx.strokeRect(this.pX, this.pY, this.width, this.height);
	this.ctx.restore();

	this.ctx.save();
	this.ctx.beginPath();
	this.ctx.fillStyle = this.offColor
	this.ctx.fillRect(this.pX + 1, this.pY + 1, this.width - 2, this.height - 2);
	this.ctx.restore();

	this.ctx.save();
	this.ctx.beginPath();
	this.ctx.fillStyle = 'RGB(204,204,204)';
	this.ctx.fillRect(this.pX + 1, this.pY + this.height/2, this.width - 2, this.height / 2 - 1);
	this.ctx.restore();

	this.checked = false;
};

// 状態１のときの描写を行うメソッドです
Switch.prototype.draw1 = function() {
	// 前のスイッチを除去
	this.ctx.clearRect(this.pX, this.pY, this.width, this.height);

	this.ctx.save();
	this.ctx.beginPath();
	this.ctx.lineWidth = 1;
	this.ctx.strokeRect(this.pX, this.pY, this.width, this.height);
	this.ctx.restore();

	this.ctx.save();
	this.ctx.beginPath();
	this.ctx.lineWidth = 1;
	this.ctx.fillStyle = this.onColor
	this.ctx.fillRect(this.pX + 1, this.pY + 1, this.width - 2, this.height - 2);
	this.ctx.restore();

	this.ctx.save();
	this.ctx.beginPath();
	this.ctx.fillStyle = 'RGB(204,204,204)';
	this.ctx.shadowBlur = 5;
	this.ctx.shadowColor = 'grey';
	this.ctx.shadowOffsetY = 5;
	this.ctx.fillRect(this.pX + 1, this.pY + 1, this.width - 2, this.height / 2 -1);
	this.ctx.restore();

	this.checked = true;
};


/*
*
* ========================================================================================
*
*   Switch2    スイッチを描画するクラスのコンストラクタ
*
*   ctx         描画するcanvas要素のコンテクスト
*   pX          描画するオブジェクトの右上のX座標
*   pY          描画するオブジェクトの右上のY座標
*   width		描画するオブジェクトの横幅
*   height      描画するオブジェクトの高さ
*/
function Switch2(pX, pY, width, height, ctx) {
	this.pX = pX;
	this.pY = pY;
	this.height = height;
	this.width = width;
	this.ctx = ctx;

	this.checked = false;
	this.clicked = false;
	this.onColor = 'rgb(0, 255, 0)';
	this.offColor = 'rgb(0, 0, 0)';
}

Switch2.prototype.setOnColor = function(onColor) {
	this.onColor = onColor;
};

Switch2.prototype.setOffColor = function(offColor) {
	this.offColor = offColor;
};

// イベントリスナー
Switch2.prototype.onClick = function(e) {
	var rect = e.target.getBoundingClientRect();
	mouseX = Math.floor(e.clientX - rect.left);
	mouseY = Math.floor(e.clientY - rect.top);
	if (((mouseX > this.pX) && (mouseX < this.pX + this.width)) && ((mouseY > this.pY) && (mouseY < this.pY + this.height))) {
		this.clicked = true;
		if (this.checked) {
			this.draw0();
		} else {
			this.draw1();
		}
	}
};

Switch2.prototype.onMouseUp = function(e) {
	var rect = e.target.getBoundingClientRect();
	mouseX = Math.floor(e.clientX - rect.left);
	mouseY = Math.floor(e.clientY - rect.top);
	if (((mouseX > this.pX) && (mouseX < this.pX + this.width)) && ((mouseY > this.pY) && (mouseY < this.pY + this.height))) {
		this.clicked = false;
	}
};

// 状態０のときの描写を行うメソッドです
Switch2.prototype.draw0 = function() {
	// 前のスイッチを除去
	this.ctx.clearRect(this.pX, this.pY, this.width, this.height);

	this.ctx.save();
	this.ctx.beginPath();
	this.ctx.lineWidth = 1;
	this.ctx.strokeRect(this.pX, this.pY, this.width, this.height);
	this.ctx.restore();

	this.ctx.save();
	this.ctx.beginPath();
	//this.ctx.fillStyle = this.offColor
	var grad = this.ctx.createLinearGradient(this.pX + 1, this.pY + 1, this.pX + 1, this.pY + this.height - 2);
	grad.addColorStop(0,'rgb(255, 255, 255)')
	grad.addColorStop(0.1,'rgb(255, 255, 255)');
	grad.addColorStop(1,'slategrey');
	this.ctx.fillStyle = grad;
	this.ctx.fillRect(this.pX + 1, this.pY + 1, this.width - 2, this.height - 2);
	this.ctx.restore();

	this.ctx.save();
	this.ctx.beginPath();
	this.ctx.fillStyle = this.offColor;
	this.ctx.fillRect(this.pX + this.width/5, this.pY + this.height/4, this.width * 3/5, this.height /10);
	this.ctx.restore();

	this.checked = false;


};

// 状態１のときの描写を行うメソッドです
Switch2.prototype.draw1 = function() {
	// 前のスイッチを除去
	this.ctx.clearRect(this.pX, this.pY, this.width, this.height);

	this.ctx.save();
	this.ctx.beginPath();
	this.ctx.lineWidth = 1;
	this.ctx.strokeRect(this.pX, this.pY, this.width, this.height);
	this.ctx.restore();

	this.ctx.save();
	this.ctx.beginPath();
	var grad = this.ctx.createLinearGradient(this.pX + 1, this.pY + 1, this.pX + 1, this.pY + this.height - 2);
	grad.addColorStop(1,'silver')
	grad.addColorStop(0.9,'silver')
	grad.addColorStop(0.3,'slategrey');
	grad.addColorStop(0,'slategrey');
	this.ctx.fillStyle = grad;
	this.ctx.fillRect(this.pX + 1, this.pY + 1, this.width - 2, this.height - 2);
	this.ctx.restore();

	this.ctx.save();
	this.ctx.beginPath();
	this.ctx.fillStyle = this.onColor;
	this.ctx.fillRect(this.pX + this.width/5, this.pY + this.height/4, this.width * 3/5, this.height /10);
	this.ctx.restore();

	this.checked = true;
};


/*
*
* ========================================================================================
*
*   Switch3     スイッチを描画するクラスのコンストラクタ
*
*   ctx         描画するcanvas要素のコンテクスト
*   pX          描画するオブジェクトの右上のX座標
*   pY          描画するオブジェクトの右上のY座標
*   width       描画するオブジェクトの横幅
*   height      描画するオブジェクトの高さ
*
*/
function Switch3(pX, pY, width, height, ctx) {
	this.pX = pX;
	this.pY = pY;
	this.height = height;
	this.width = width;
    this.radius = width / 2;
	this.ctx = ctx;

	this.checked = false;
	this.onColor = 'rgba(0, 0, 255, 0.5)';
	this.offColor = 'rgba(255, 255, 255, 0.5)';
    this.strokeColor = {};
}

//オン時の色を設定するメソッド
Switch3.prototype.setOnColor = function(onColor) {
	this.onColor = onColor;
};

//オフ時の色を設定するメソッド
Switch3.prototype.setOffColor = function(offColor) {
	this.offColor = offColor;
};

//オフ時の背景色を設定するメソッド
Switch3.prototype.setBGColor = function(bgColor) {
	this.bgColor = bgColor;
};

// Switch3 の strokestyle を設定するメソッド
Switch3.prototype.setStrokeColor = function(strokeColor) {
	this.strokeColor = strokeColor;
};


// クリックイベントリスナー
Switch3.prototype.onClick = function(e, item, form) {
	var rect = e.target.getBoundingClientRect();
	mouseX = Math.floor(e.clientX - rect.left);
	mouseY = Math.floor(e.clientY - rect.top);
	if (((mouseX > this.pX) && (mouseX < this.pX + this.width)) && ((mouseY > this.pY) && (mouseY < this.pY + this.height))) {
		this.clicked = true;
        if (this.checked) { //このオブジェクトがクリックされた場合、formの値を書き換え送信
			this.draw0();
		} else {
			this.draw1();
		}
	}
};

// 状態０のときの描写を行うメソッドです
Switch3.prototype.draw0 = function() {
	// 前のスイッチを除去
	this.ctx.clearRect(this.pX, this.pY, this.width, this.height);
    
    // 外枠の矩形を描画します
	this.ctx.save();
	this.ctx.lineWidth = 1;
    this.ctx.strokeStyle = this.strokeColor.outLine;
    this.ctx.fillStyle = this.bgColor;
	this.ctx.fillRect(this.pX, this.pY, this.width, this.height);	
    this.ctx.restore();
    
    // 内側の円を描画します 1
	this.ctx.save();
	this.ctx.beginPath();
	this.ctx.lineWidth = 1;
    this.ctx.strokeStyle = this.strokeColor.inLine0;
	this.ctx.arc(this.pX + this.radius, this.pY + this.radius, this.radius - 4, 0, Math.PI * 2, true);
    this.ctx.fillStyle = this.offColor;
    this.ctx.fill();
	this.ctx.stroke();
	this.ctx.restore();

    // 内側の円を描画します 2
	this.ctx.save();
	this.ctx.beginPath();
	this.ctx.lineWidth = 1;
    this.ctx.strokeStyle = this.strokeColor.inLine1;
	this.ctx.arc(this.pX + this.radius, this.pY + this.radius, this.radius - 6, 0, Math.PI * 2, true);
    this.ctx.fillStyle = this.offColor;
    this.ctx.fill();
	this.ctx.stroke();
	this.ctx.restore();

    // 「|」を描画します
	this.ctx.save();
	this.ctx.lineWidth = 2;
    this.ctx.strokeStyle = this.onColor;
    this.ctx.fillStyle = this.onColor;
	this.ctx.fillRect(this.pX + this.radius - 1, this.pY + 1, 2, 8);	
    this.ctx.restore();    
    
    this.checked = false;
};

// 状態１のときの描写を行うメソッドです
Switch3.prototype.draw1 = function() {
	// 前のスイッチを除去
	this.ctx.clearRect(this.pX, this.pY, this.width, this.height);

    // 外枠の矩形を描画します
	this.ctx.save();
	this.ctx.lineWidth = 1;
    this.ctx.strokeStyle = this.strokeColor.outLine;
    this.ctx.fillStyle = this.bgColor;
	this.ctx.fillRect(this.pX, this.pY, this.width, this.height);	
    this.ctx.restore();
    
    // 内側の円を描画します 1
	this.ctx.save();
	this.ctx.beginPath();
	this.ctx.lineWidth = 1;
    this.ctx.strokeStyle = this.strokeColor.inLine0;
	this.ctx.arc(this.pX + this.radius, this.pY + this.radius, this.radius - 4, 0, Math.PI * 2, true);
    //this.ctx.fillStyle = "#fff";
    this.ctx.fillStyle = this.onColor;
    this.ctx.fill();
	this.ctx.stroke();
	this.ctx.restore();

    // 内側の円を描画します 2
	this.ctx.save();
	this.ctx.beginPath();
	this.ctx.lineWidth = 1;
    this.ctx.strokeStyle = this.offColor;
	this.ctx.arc(this.pX + this.radius, this.pY + this.radius, this.radius - 6, 0, Math.PI * 2, true);
    this.ctx.fillStyle = this.onColor;
    this.ctx.fill();
	this.ctx.stroke();
	this.ctx.restore();

    // 「|」を描画します
	this.ctx.save();
	this.ctx.lineWidth = 2;
    this.ctx.strokeStyle = this.onColor;
    this.ctx.fillStyle = "#ddd";
	this.ctx.fillRect(this.pX + this.radius - 1, this.pY + 2, 2, 8);	
    this.ctx.restore();    

    this.checked = true;
};


/*
*
* ========================================================================================
*
*   Switch4     スイッチを描画するクラスのコンストラクタ
*
*   ctx         描画するcanvas要素のコンテクスト
*   pX          描画するオブジェクトの右上のX座標
*   pY          描画するオブジェクトの右上のY座標
*   width       描画するオブジェクトの横幅
*   height      描画するオブジェクトの高さ
*
*/
function Switch4(pX, pY, width, height, ctx) {
	this.pX = pX;
	this.pY = pY;
	this.height = height;
	this.width = width;
    this.radius = width / 2;
	this.ctx = ctx;

	this.checked = false;
	this.onColor = 'rgba(0, 0, 255, 0.5)';
	this.offColor = 'rgba(255, 255, 255, 0.5)';
    this.strokeColor = {};
}

//オン時の色を設定するメソッド
Switch4.prototype.setOnColor = function(onColor) {
	this.onColor = onColor;
};

//オフ時の色を設定するメソッド
Switch4.prototype.setOffColor = function(offColor) {
	this.offColor = offColor;
};

//オフ時の背景色を設定するメソッド
Switch4.prototype.setBGColor = function(bgColor) {
	this.bgColor = bgColor;
};

// Switch3 の strokestyle を設定するメソッド
Switch4.prototype.setStrokeColor = function(strokeColor) {
	this.strokeColor = strokeColor;
};


// クリックイベントリスナー
Switch4.prototype.onClick = function(e, item, form) {
	var rect = e.target.getBoundingClientRect();
	mouseX = Math.floor(e.clientX - rect.left);
	mouseY = Math.floor(e.clientY - rect.top);
	if (((mouseX > this.pX) && (mouseX < this.pX + this.width)) && ((mouseY > this.pY) && (mouseY < this.pY + this.height))) {
		this.clicked = true;
        if (this.checked) { //このオブジェクトがクリックされた場合、formの値を書き換え送信
			this.draw0();
		} else {
			this.draw1();
		}
	}
};

// 状態０のときの描写を行うメソッドです
Switch4.prototype.draw0 = function() {
	// 前のスイッチを除去
	this.ctx.clearRect(this.pX, this.pY, this.width, this.height);
    
    this.ctx.strokeStyle = this.strokeColor;

    // 外枠の矩形を描画します
	this.ctx.save();
	this.ctx.lineWidth = 1;
    this.ctx.strokeStyle = this.strokeColor.outLine;
    this.ctx.fillStyle = this.bgColor;
	this.ctx.fillRect(this.pX, this.pY, this.width, this.height);	
    this.ctx.restore();
/*
	this.ctx.save();
    this.ctx.beginPath();
	this.ctx.lineWidth = 1;
    this.ctx.strokeStyle = this.strokeColor.outLine;
	this.ctx.arc(this.pX + this.radius, this.pY + this.radius, this.radius - 2, 0, Math.PI * 2, true);
    this.ctx.fillStyle = this.bgColor;
    this.ctx.fill();
    this.ctx.stroke();
	//this.ctx.fillRect(this.pX, this.pY, this.width, this.height);	
    this.ctx.restore();
*/
    
    // 内側の円を描画します 1
	this.ctx.save();
	this.ctx.beginPath();
	this.ctx.lineWidth = 1;
    this.ctx.strokeStyle = this.strokeColor.inLine0;
	this.ctx.arc(this.pX + this.radius, this.pY + this.radius, this.radius - 2, 0, Math.PI * 2, true);
    this.ctx.fillStyle = this.offColor;
    this.ctx.fill();
	this.ctx.stroke();
	this.ctx.restore();

    // 内側の円を描画します 2
	this.ctx.save();
	this.ctx.beginPath();
	this.ctx.lineWidth = 2;
    //this.ctx.strokeStyle = this.strokeColor.inLine1;
    this.ctx.strokeStyle = this.onColor;
	this.ctx.arc(this.pX + this.radius, this.pY + this.radius, this.radius - 7, 0, Math.PI * 2, true);
    this.ctx.fillStyle = this.offColor;
    this.ctx.fill();
	this.ctx.stroke();
	this.ctx.restore();


    // 「|」を描画します
	this.ctx.save();

    this.ctx.fillStyle = this.offColor;
    var rectLength = 10;
	this.ctx.fillRect(this.pX + this.radius- rectLength/2, this.pY + rectLength/2, rectLength, rectLength);    
    
	this.ctx.lineWidth = 1;
    this.ctx.strokeStyle = this.onColor;
    this.ctx.fillStyle = this.onColor;
    //this.ctx.fillStyle = this.strokeColor.inLine1;
	this.ctx.fillRect(this.pX + this.radius - 1.2, this.pY + 5, 2.4, 12);	
    this.ctx.restore();    
    
    this.checked = false;
};

// 状態１のときの描写を行うメソッドです
Switch4.prototype.draw1 = function() {
	// 前のスイッチを除去
	this.ctx.clearRect(this.pX, this.pY, this.width, this.height);

    // 外枠を描画します
/*
	this.ctx.save();
	this.ctx.beginPath();
	this.ctx.lineWidth = 1;
    this.ctx.strokeStyle = this.strokeColor.inLine1;
	this.ctx.arc(this.pX + this.radius, this.pY + this.radius, this.radius, 0, Math.PI * 2, true);
	this.ctx.stroke();
	this.ctx.restore();
*/
    
    // 内側の円を描画します 1
	this.ctx.save();
	this.ctx.beginPath();
	this.ctx.lineWidth = 1;
    this.ctx.strokeStyle = this.strokeColor.inLine0;
	this.ctx.arc(this.pX + this.radius, this.pY + this.radius, this.radius - 2, 0, Math.PI * 2, true);
    this.ctx.fillStyle = this.onColor;
    this.ctx.fill();
	this.ctx.stroke();
	this.ctx.restore();

    // 内側の円を描画します 2
	this.ctx.save();
	this.ctx.beginPath();
	this.ctx.lineWidth = 2;
    this.ctx.strokeStyle = this.strokeColor.inLine2;
	this.ctx.arc(this.pX + this.radius, this.pY + this.radius, this.radius - 7, 0, Math.PI * 2, true);
    this.ctx.fillStyle = this.onColor;
    this.ctx.fill();
	this.ctx.stroke();
	this.ctx.restore();

    // 「|」を描画します
	this.ctx.save();

    this.ctx.fillStyle = this.onColor;
    var rectLength = 10;
	this.ctx.fillRect(this.pX + this.radius- rectLength/2, this.pY + rectLength/2, rectLength, rectLength);    
    
	this.ctx.lineWidth = 1;
    this.ctx.strokeStyle = this.offColor;
    this.ctx.fillStyle = this.offColor;
	this.ctx.fillRect(this.pX + this.radius - 1.2, this.pY + 5, 2.4, 12);	
    this.ctx.restore();    

    this.checked = true;
};


/*
*
* ========================================================================================
*
*   SeekBar     シークバーを描画するクラス
*
*   hength      描画するオブジェクトの横の長さ
*   height      描画するオブジェクトの縦の高さ
*   minimam     シークバーの最小値
*   maximam     シークバーの最大値
*
*/
function SeekBar(length, height, minimam, maximam) {
	this.length = length;
	this.height = height;
	this.minimam = minimam;
	this.maximam = maximam;

	// デフォルト値の設定
	this.value = Math.round((this.maximam - this.minimam) / 2);
	this.clicked = false;
	this.colorTrans = false;
    this.drawingBG = false;
}

//値を設定するメソッド
SeekBar.prototype.setValue = function(value) {
	if(value > this.maximam){
		this.value = this.maximam;
	}else if(value < this.minimam){
		this.value = this.minimam;
	}else{
		this.value = value;
	}
};

//色を設定するメソッド
SeekBar.prototype.setColor = function(color) {
	this.color = color;
};

SeekBar.prototype.setColor2 = function(color){
	this.color2 = color;
}

SeekBar.prototype.setColor3 = function(color){
	this.color3 = color;
}

// ストロークの色を設定します
SeekBar.prototype.setStrokeColor = function(color){
	this.strokeColor = color;
}

// 背景色を描画するか否かを設定します
SeekBar.prototype.drawingBG = function(bl){
	this.drawingBG = bl;
}

// 背景色を設定します
SeekBar.prototype.setBGColor = function(color){
	this.bgColor = color;
}

//マウスが押されたときに呼ばれるメソッド
SeekBar.prototype.onMouseDown = function(e) {
	var rect = e.target.getBoundingClientRect();
	mouseX = Math.floor(e.clientX - rect.left);
	mouseY = Math.floor(e.clientY - rect.top);
	if (((mouseX > this.knobX) && (mouseX < this.knobX + 10)) && ((mouseY > this.knobY) && (mouseY < this.knobY + 10 + this.height))) {
		this.clicked = true;
	} else {
		this.clicked = false;
	}
	if((mouseX > this.pX) && (mouseX < this.pX + this.length) && (mouseY > this.pY) && (mouseY < this.pY + this.height)){
		this.value = (mouseX - this.pX) / this.length * (this.maximam - this.minimam) + this.minimam;
		this.draw(this.ctx, this.pX, this.pY, this.bgColor);
		this.clicked = true;
	}
};

//マウスが放されたときに呼ばれるメソッド
SeekBar.prototype.onMouseUp = function(e) {
	this.clicked = false;
};

//マウスが動かされたときに呼ばれるメソッド
SeekBar.prototype.onMouseMove = function(e) {
	if (this.clicked) {
		var rect = e.target.getBoundingClientRect();
		mouseX = Math.floor(e.clientX - rect.left);
		if (mouseX >= this.pX && mouseX <= this.pX + this.length) {
			this.value = (mouseX - this.pX) / this.length * (this.maximam - this.minimam) + this.minimam;
			this.draw(this.ctx, this.pX, this.pY, this.bgColor);
		} else {

		}
	}
};

//タッチされたときに呼ばれるメソッド
SeekBar.prototype.onTouchStart = function(e) {
	var rect = e.target.getBoundingClientRect();
	mouseX = Math.floor(e.clientX - rect.left);
	mouseY = Math.floor(e.clientY - rect.top);
	if (((mouseX > this.knobX - 15) && (mouseX < this.knobX + 30)) && ((mouseY > this.knobY) && (mouseY < this.knobY + 10 + this.height))) { //シークバーの持ち手がクリックされている場合
		this.clicked = true;
	} else {
		this.clicked = false;
	}
	if((mouseX > this.pX) && (mouseX < this.pX + this.length) && (mouseY > this.pY) && (mouseY < this.pY + this.height)){
		this.value = (mouseX - this.pX) / this.length * (this.maximam - this.minimam) + this.minimam;
		this.draw(this.ctx, this.pX, this.pY, this.bgColor);
		this.clicked = true;
	}
};

//タッチが終わった時に呼ばれるメソッド
SeekBar.prototype.onTouchEnd = function(e) {
	this.clicked = false;
};

//マウスが動かされたときに呼ばれるメソッド
SeekBar.prototype.onTouchMove = function(e) {
	if (this.clicked) {
		var rect = e.target.getBoundingClientRect();
		mouseX = Math.floor(e.clientX - rect.left);
		if (mouseX >= this.pX && mouseX <= this.pX + this.length) { //ポインタがシークバー内にあるかチェック
			this.value = (mouseX - this.pX) / this.length * (this.maximam - this.minimam) + this.minimam;
			this.draw(this.ctx, this.pX, this.pY, this.bgColor);
		} else {

		}
	}
};

SeekBar.prototype.draw = function(ctx, pX, pY, bgColor) {
	this.pX = pX;
	this.pY = pY;
	this.ctx = ctx;

	this.clearOffsetX = 8;
	this.clearOffsetY = 8;
	this.clearOffsetLength = 18;
	this.clearOffsetHeight = 22;

	ctx.clearRect(pX - this.clearOffsetX, pY - this.clearOffsetY, this.length + this.clearOffsetLength, this.height + this.clearOffsetHeight);

	// 外側の矩形を描画
	ctx.save();
    // dummy
	//ctx.lineWidth = 2;
	ctx.lineWidth = 1;
    ctx.strokeStyle = this.strokeColor;
	ctx.strokeRect(pX, pY, this.length, this.height);
    if (this.drawingBG) { // 背景色が指定されている場合
        ctx.fillStyle = this.bgColor;
        ctx.fillRect(pX + 1, pY + 1, this.length, this.height - 2);
    }
    ctx.restore();

	// 内側の矩形を描画
	ctx.save();
	var innerLength = this.value / this.maximam * (this.length - 2);

	if(this.colorTrans){
		if(this.value > this.maximam*0.9){
			ctx.fillStyle = this.color3;
		}else if(this.value > this.maximam*0.7){
			ctx.fillStyle = this.color2;
		}else{
			ctx.fillStyle = this.color;
		}
	}else{
		ctx.fillStyle = this.color;
	}
	ctx.fillRect(pX + 1, pY + 1, innerLength, this.height - 2);
	ctx.restore();

	// つまみを描画
	this.knobX = pX + (this.value - this.minimam) / (this.maximam - this.minimam) * (this.length - 2) - 5;
	this.knobY = pY - 5;
	ctx.save();
	ctx.fillStyle = 'rgba(200, 200, 200, 0.9)';
	ctx.shadowBlur = 4;
	ctx.shadowColor = '#003377';
	ctx.shadowOffsetX = 3;
	ctx.shadowOffsetY = 3;
	ctx.fillRect(this.knobX, this.knobY, 10, this.height + 10);
	ctx.restore();
};


/*
*
* ========================================================================================
*
*   VolumnButton    値を増減する+,-ボタンを描画するクラス
*
*/
function VolumnButton(ctx, pX, pY, width, height, direction){
	this.ctx = ctx;
	this.pX = pX;
	this.pY = pY;
	this.width = width;
	this.height = height;
	this.direction = direction;
	this.clicked = false;
	this.timerId = 0;
};

// ボタンの色を設定します
VolumnButton.prototype.setColor = function(color){
	this.color = color;
};

// 背景色を設定します
VolumnButton.prototype.setBGColor = function(color){
	this.bgColor = color;
};

// ストローク色を設定します
VolumnButton.prototype.setStrokeColor = function(color){
	this.strokeColor = color;
};

VolumnButton.prototype.onMouseDown = function(e, item) {
	var rect = e.target.getBoundingClientRect();
	mouseX = Math.floor(e.clientX - rect.left);
	mouseY = Math.floor(e.clientY - rect.top);
	if (((mouseX > this.pX) && (mouseX < this.pX + this.width)) && ((mouseY > this.pY) && (mouseY < this.pY + this.height))) {
		this.clicked = true;
		if(this.direction == "plus"){
			if(item.value < item.maximam) item.setValue(parseInt(item.value) + 1);
			item.draw(item.ctx, item.pX, item.pY);
			this.timerId = setInterval(function(){
				if(item.value < item.maximam){
					item.setValue(item.value + 1);
					item.draw(item.ctx, item.pX, item.pY);
				}
			}, 200);
		}else{
			if(item.value > item.minimam) item.setValue(parseInt(item.value) - 1);
			item.draw(item.ctx, item.pX, item.pY);
			this.timerId = setInterval(function(){
				if(item.value > item.minimam){
					item.setValue(item.value - 1);
					item.draw(item.ctx, item.pX, item.pY);
				}
			}, 200);
		}

		this.ctx.save();
		this.ctx.beginPath();
		this.ctx.fillStyle = 'rgba(153, 153, 153, 0.4)';
        this.ctx.arc(this.pX + this.width/2, this.pY + this.height/2, this.width/2 - 1, 0, Math.PI * 2, true);
        this.ctx.fill();
		this.ctx.restore();

	}
};

VolumnButton.prototype.onMouseUp = function(e){
	if (((mouseX > this.pX) && (mouseX < this.pX + this.width)) && ((mouseY > this.pY) && (mouseY < this.pY + this.height))) {
		this.clicked = false;
		clearInterval(this.timerId);
		this.draw();
	}
}

VolumnButton.prototype.draw = function(){
    this.ctx.clearRect(this.pX, this.pY, this.width, this.height);

    this.ctx.save();
	this.ctx.beginPath();
    // dummy
	//this.ctx.lineWidth = 2;
    this.ctx.strokeStyle = this.strokeColor;
	this.ctx.arc(this.pX + this.width/2, this.pY + this.height/2, this.width/2, 0, Math.PI * 2, true);
	this.ctx.stroke();
	this.ctx.restore();

	if(this.direction == "plus"){
		this.ctx.save();
		this.ctx.fillStyle = this.color;
        this.ctx.strokeStyle = this.color;
        this.ctx.fillRect(this.pX + this.width / 2 - 1.5, this.pY + 2, 3, this.height - 4);
        this.ctx.fillRect(this.pX + 2, this.pY + this.width / 2 - 1.5, this.width - 4, 3);
        // dummy
		//this.ctx.fill();
		this.ctx.restore();
	}else{
		this.ctx.save();
		this.ctx.fillStyle = this.color;
        this.ctx.strokeStyle = this.color;
        this.ctx.fillRect(this.pX + 2, this.pY + this.width / 2 - 1.5, this.width - 4, 3);
        // dummy
		//this.ctx.fill();
		this.ctx.restore();
	}
};


/*
*
* ========================================================================================
*
*   PlusMinusButton    値を増減するボタンを描画するクラスです
*
*/
function PlusMinusButton(ctx, pX, pY, width, height, code){
	this.ctx = ctx;
	this.pX = pX;
	this.pY = pY;
	this.width = width;
	this.height = height;
	this.code = code;
	this.color = 'rgb(0, 256, 0)';
	this.clicked = false;
	this.timerId = 0;
};

// ボタンの色を設定します
PlusMinusButton.prototype.setColor = function(color){
	this.color = color;
};

// 背景色を設定します
PlusMinusButton.prototype.setBGColor = function(color){
	this.bgColor = color;
};

// ストローク色を設定します
PlusMinusButton.prototype.setStrokeColor = function(color){
	this.strokeColor = color;
};

// マウス・ダウン時のエフェクトを設定します
PlusMinusButton.prototype.onMouseDownEffect = function() {
    
		this.ctx.save();
		this.ctx.beginPath();
		this.ctx.lineWidth = 1;
		this.ctx.fillStyle = 'rgba(153, 153, 153, 0.4)';
		this.ctx.fillRect(this.pX, this.pY, this.width, this.height);
		this.ctx.restore();
};

PlusMinusButton.prototype.onMouseDown = function(e, item) {
	var rect = e.target.getBoundingClientRect();
	mouseX = Math.floor(e.clientX - rect.left);
	mouseY = Math.floor(e.clientY - rect.top);
	if (((mouseX > this.pX) && (mouseX < this.pX + this.width)) && ((mouseY > this.pY) && (mouseY < this.pY + this.height))) {
		this.clicked = true;
		if(this.code == "plus"){
			if(item.value < item.maximam) item.setValue(parseInt(item.value) + 1);
			item.draw(item.ctx, item.pX, item.pY);
			this.timerId = setInterval(function(){
				if(item.value < item.maximam){
					item.setValue(item.value + 1);
					item.draw(item.ctx, item.pX, item.pY);
				}
			}, 200);
		}else{
			if(item.value > item.minimam) item.setValue(parseInt(item.value) - 1);
			item.draw(item.ctx, item.pX, item.pY);
			this.timerId = setInterval(function(){
				if(item.value > item.minimam){
					item.setValue(item.value - 1);
					item.draw(item.ctx, item.pX, item.pY);
				}
			}, 200);
		}
        this.onMouseDownEffect();
	}
};

PlusMinusButton.prototype.onMouseUp = function(e){
	if (((mouseX > this.pX) && (mouseX < this.pX + this.width)) && ((mouseY > this.pY) && (mouseY < this.pY + this.height))) {
		this.clicked = false;
		clearInterval(this.timerId);
		this.ctx.clearRect(this.pX, this.pY, this.width, this.height);
		this.draw();
	}
}

// ボタンの縁を描画します
PlusMinusButton.prototype.drawEdge = function(){
	this.ctx.save();
	this.ctx.beginPath();
	this.ctx.lineWidth = 1;
	this.ctx.strokeRect(this.pX, this.pY, this.width, this.height);
	this.ctx.restore();
};

// ”＋” 側のボタンを描画します
PlusMinusButton.prototype.drawPlus = function(){
    //this.ctx.save();
    //this.ctx.font = "8pt Arial";
    this.ctx.fillStyle = "#fff";
    //this.ctx.fillText("+", this.px + this.width / 2, this.py + this.height / 2, this.width);
    this.ctx.strokeText("+", this.px, this.py, this.width);
    //this.ctx.restore();
};

// ”-” 側のボタンを描画します
PlusMinusButton.prototype.drawMinus = function(){
};

PlusMinusButton.prototype.draw = function(){

    this.drawEdge();
    
	if(this.code == "plus"){
        this.drawPlus();
	}else{
        this.drawMinus();
	}
};



/*
*
* ========================================================================================
*
*   ArrowButton    値を増減する矢印ボタンを描画するクラスです
*
*/

function ArrowButton(ctx, pX, pY, width, height, code) {
 
    PlusMinusButton.apply(this);
}

ArrowButton.prototype = Object.create(PlusMinusButton.prototype);
ArrowButton.prototype.constructor = ArrowButton;

// ボタンの縁を描画します
ArrowButton.prototype.drawEdge = function(){
	this.ctx.save();
	this.ctx.beginPath();
	this.ctx.lineWidth = 1;
	this.ctx.strokeRect(this.pX, this.pY, this.width, this.height);
	this.ctx.restore();
};


// ”→” 側のボタンを描画します
ArrowButton.prototype.drawPlus = function(){
    this.ctx.save();
    this.ctx.fillStyle = this.color;
    this.ctx.beginPath();
    this.ctx.moveTo(this.pX + this.width/10, this.pY + this.height/10);
    this.ctx.lineTo(this.pX + this.width/10, this.pY + this.height - this.height/10);
    this.ctx.lineTo(this.pX + this.width - this.width/10, this.pY + this.height/2);
    this.ctx.closePath();
    this.ctx.fill();
    this.ctx.restore();
};

// ”←” 側のボタンを描画します
ArrowButton.prototype.drawMinus = function(){
    this.ctx.save();
    this.ctx.fillStyle = this.color;
    this.ctx.beginPath();
    this.ctx.moveTo(this.pX + this.width - this.width/10, this.pY + this.height/10);
    this.ctx.lineTo(this.pX + this.width - this.width/10, this.pY + this.height - this.height/10);
    this.ctx.lineTo(this.pX + this.width/10, this.pY + this.height/2);
    this.ctx.closePath();
    this.ctx.fill();
    this.ctx.restore();
};



/*
*
* ========================================================================================
*
*   VerticalSeekBar    縦方向のシークバーを描画するクラス
*
*/
function VerticalSeekBar(length, width, minimam, maximam) {
	this.length = length;
	this.width = width;
	this.minimam = minimam;
	this.maximam = maximam;

	// デフォルト値の設定
	this.value = Math.round((this.maximam - this.minimam) / 2);
}

VerticalSeekBar.prototype.setValue = function(value) {
	if(value > this.maximam){
		this.value = this.maximam;
	}else if(value < this.minimam){
		this.value = this.minimam;
	}else{
		this.value = value;
	}
};

VerticalSeekBar.prototype.setColor = function(color) {
	this.color = color;
};

VerticalSeekBar.prototype.onTouchStart = function(e) {
	var rect = e.target.getBoundingClientRect();
	mouseX = Math.floor(e.clientX - rect.left);
	mouseY = Math.floor(e.clientY - rect.top);
	if (((mouseX > this.knobX) && (mouseX < this.knobX + 10 + this.width)) && ((mouseY > this.knobY - 15) && (mouseY < this.knobY + 30))) {
		this.clicked = true;
	} else {
		this.clicked = false;
	}
};

VerticalSeekBar.prototype.onTouchEnd = function(e) {
	this.clicked = false;
};

VerticalSeekBar.prototype.onTouchMove = function(e) {
	if (this.clicked) {
		var rect = e.target.getBoundingClientRect();
		mouseY = Math.floor(e.clientY - rect.top);
		if (mouseY >= this.pY && mouseY <= this.pY + this.length) {
			this.value = (this.length - (mouseY - this.pY)) / this.length * (this.maximam - this.minimam) + this.minimam;
			this.draw(this.ctx, this.pX, this.pY);
		} else {

		}
	}
};

VerticalSeekBar.prototype.onMouseDown = function(e) {
	var rect = e.target.getBoundingClientRect();
	mouseX = Math.floor(e.clientX - rect.left);
	mouseY = Math.floor(e.clientY - rect.top);
	if (((mouseX > this.knobX) && (mouseX < this.knobX + 10 + this.width)) && ((mouseY > this.knobY) && (mouseY < this.knobY + 10))) {
		this.clicked = true;
	} else {
		this.clicked = false;
	}
};

VerticalSeekBar.prototype.onMouseUp = function(e) {
	var rect = e.target.getBoundingClientRect();
	mouseX = Math.floor(e.clientX - rect.left);
	mouseY = Math.floor(e.clientY - rect.top);
	if (((mouseX > this.knobX) && (mouseX < this.knobX + 10 + this.width)) && ((mouseY > this.knobY) && (mouseY < this.knobY + 10))) {
		this.clicked = false;
	}
};

VerticalSeekBar.prototype.onMouseMove = function(e) {
	if (this.clicked) {
		var rect = e.target.getBoundingClientRect();
		mouseY = Math.floor(e.clientY - rect.top);
		if (mouseY >= this.pY && mouseY <= this.pY + this.length) {
			this.value = (this.length - (mouseY - this.pY)) / this.length * (this.maximam - this.minimam) + this.minimam;
			this.draw(this.ctx, this.pX, this.pY);
		} else {

		}
	}
};

VerticalSeekBar.prototype.draw = function(ctx, pX, pY) {
	this.pX = pX;
	this.pY = pY;
	this.ctx = ctx;

	this.clearOffsetX = 7;
	this.clearOffsetY = 6;
	this.clearOffsetWidth = 20;
	this.clearOffsetLength = 20;

	ctx.clearRect(pX - this.clearOffsetX, pY - this.clearOffsetY, this.width + this.clearOffsetWidth, this.length + this.clearOffsetLength);

	// 外側の矩形を描画
	ctx.save();
	ctx.lineWidth = 1;
	ctx.strokeRect(pX, pY, this.width, this.length);
	ctx.restore();is

	// 内側の矩形を描画
	ctx.save();
	var innerLength = (this.value - this.minimam) / (this.maximam - this.minimam) * (this.length - 2);
	ctx.fillStyle = this.color;
	ctx.fillRect(pX + 1, pY + (this.length - innerLength) - 1, this.width - 2, innerLength);
	ctx.restore();

	// つまみを描画
	this.knobX = pX - 5;
	this.knobY = pY + (this.length - innerLength) - 5;
	ctx.save();
	ctx.fillStyle = 'rgba(200, 200, 200, 0.9)';
	ctx.shadowBlur = 4;
	ctx.shadowColor = '#003377';
	ctx.shadowOffsetX = 3;
	ctx.shadowOffsetY = 3;
	ctx.fillRect(this.knobX, this.knobY, this.width + 10, 10);
	ctx.restore();

};

function VerticalProgressBar(length, width, minimam, maximam) {
    this.length = length;
    this.width = width;
    this.minimam = minimam;
    this.maximam = maximam;

    // デフォルト値を設定
    this.color = 'RGB(0,255,51)';
    this.value = this.maximam / 2;
}

VerticalProgressBar.prototype.setValue = function(value) { // 値を設定するメソッド
    if(value > this.maximam){
        this.value = this.maximam;
    }else if(value < this.minimam){
        this.value = this.minimam;
    }else{
        this.value = value;
    }
};

VerticalProgressBar.prototype.setColor = function(color) { // 色を設定するメソッド
    this.color = color;
};

VerticalProgressBar.prototype.setScale = function(font, unit) { // 目盛を設定するメソッド
    this.font = font;
    this.unit = unit;
    this.scale = true; // このメソッドが呼ばれた場合のみ目盛を描画
};

VerticalProgressBar.prototype.stroke = function(ctx, pX, pY) { // 描画するメソッド
    // 外側の矩形を描画
    ctx.save();
    ctx.lineWidth = 2;
    ctx.strokeRect(pX, pY, this.width, this.length);
    ctx.restore();

    // 内側の矩形を描画
    ctx.save();
    var innerLength = (this.value - this.minimam) / (this.maximam - this.minimam) * (this.length - 2);
    ctx.fillStyle = this.color;
    ctx.fillRect(pX + 1, pY + (this.length - innerLength) - 1, this.width - 2, innerLength);
    ctx.restore();

    // 目盛を描画
    if (this.scale) {
        lineLength = 5;
        offsetX = 5;
        offsetY = 3;

        ctx.save();
        ctx.beginPath();
        ctx.lineWidth = 2;
        ctx.moveTo(pX + this.width, pY);
        ctx.lineTo(pX + this.width + lineLength, pY);
        ctx.moveTo(pX + this.width, pY + Math.round(this.length / 2));
        ctx.lineTo(pX + this.width + lineLength, pY + Math.round(this.length / 2));
        ctx.moveTo(pX + this.width, pY + this.length);
        ctx.lineTo(pX + this.width + lineLength, pY + this.length);
        ctx.stroke();
        ctx.restore();

        ctx.font = this.font;
        ctx.fillText(this.maximam + this.unit, pX + this.width + lineLength, pY + offsetY);
        ctx.fillText((this.maximam - this.minimam) / 2 + this.minimam + this.unit, pX + this.width + lineLength, pY + Math.round(this.length / 2) + offsetY);
        ctx.fillText(this.minimam + this.unit, pX + this.width + lineLength, pY + this.length + offsetY);
    }
};